@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2011 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,6 +21,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <time.h>
 
 #include "wx/wx.h"
 #include "wx/wfstream.h"
@@ -41,6 +42,9 @@
 #endif
 
 extern void FindPhonemesUsed(void);
+extern void DisplayErrorFile(const char *fname);
+extern int utf8_out(unsigned int c, char *buf);
+extern void DrawEnvelopes();
 char path_source[sizeof(path_home)+20];
 
 
@@ -87,34 +91,39 @@ static keywtab_t k_conditions[] = {
 	{"OR",       0,  k_OR},
 	{"THEN",     0,  k_THEN},
 	{"prevPh",   tWHICH_PHONEME,  0x000},
+	{"prevPhW",  tWHICH_PHONEME,  0x500},
 	{"thisPh",   tWHICH_PHONEME,  0x100},
 	{"nextPh",   tWHICH_PHONEME,  0x200},
 	{"next2Ph",  tWHICH_PHONEME,  0x300},
 	{"nextPhW",  tWHICH_PHONEME,  0x400},
-	{"prevPhW",  tWHICH_PHONEME,  0x500},
 	{"nextVowel",tWHICH_PHONEME,  0x600},
+//	{"next2PhW", tWHICH_PHONEME,  0x700},
 
 //	{"numVowels",    tTEST,  0x000},
 //	{"afterStress",  tTEST,  0x000},
 
 	{"PreVoicing",   tTEST,  0xf01},
 	{"KlattSynth",   tTEST,  0xf02},
+	{"MbrolaSynth",  tTEST,  0xf03},
 	{NULL, 0, 0}
 };
 
 
-// place of articulation
-#define i_isVel      0x28
 
 static keywtab_t k_properties[] = {
 	{"isPause",      0,  phPAUSE},
 	{"isVowel",      0,  phVOWEL},
 	{"isNasal",      0,  phNASAL},
 	{"isLiquid",     0,  phLIQUID},
+	{"isUStop",      0,  phSTOP},
 	{"isVFricative", 0,  phVFRICATIVE},
 
 	{"isPalatal",    0,  i_isPalatal},
 	{"isRhotic",     0,  i_isRhotic},
+	{"isSibilant",   0,  i_isSibilant},
+	{"isFlag1",      0,  i_isFlag1},
+	{"isFlag2",      0,  i_isFlag2},
+	{"isFlag3",      0,  i_isFlag3},
 
 	{"isVel",        0,  i_isVel},
 
@@ -133,6 +142,10 @@ static keywtab_t k_properties[] = {
 	{"isNotVowel",   0,  i_isNotVowel},
 	{"isFinalVowel", 0,  i_isFinalVowel},
 	{"isVoiced",     0,  i_isVoiced},   // voiced consonant, or vowel
+	{"isFirstVowel", 0,  i_isFirstVowel},
+	{"isSecondVowel", 0, i_isSecondVowel},
+	{"isSeqFlag1",   0,  i_isSeqFlag1},
+	{"isTranslationGiven", 0, i_IsTranslationGiven},   // phoneme translation given in **_list or as [[...]]
 
 	{NULL, 0, 0}
 };
@@ -142,6 +155,7 @@ enum {
 	kPROCEDURE,
 	kENDPHONEME,
 	kENDPROCEDURE,
+	kEQUIVALENTS,
 	kPHONEMENUMBER,
 	kPHONEMETABLE,
 	kINCLUDE,
@@ -177,6 +191,35 @@ enum {
 };
 
 
+enum {
+	kTUNE = 1,
+	kENDTUNE,
+	kTUNE_PREHEAD,
+	kTUNE_ONSET,
+	kTUNE_HEAD,
+	kTUNE_HEADENV,
+	kTUNE_HEADEXTEND,
+	kTUNE_HEADLAST,
+	kTUNE_NUCLEUS0,
+	kTUNE_NUCLEUS1,
+	kTUNE_SPLIT,
+};
+
+static keywtab_t k_intonation[] = {
+	{"tune",      0,   kTUNE},
+	{"endtune",   0,   kENDTUNE},
+	{"prehead",   0,   kTUNE_PREHEAD},
+	{"onset",     0,   kTUNE_ONSET},
+	{"head",      0,   kTUNE_HEAD},
+	{"headenv",   0,   kTUNE_HEADENV},
+	{"headextend", 0,  kTUNE_HEADEXTEND},
+	{"headlast",  0,   kTUNE_HEADLAST},
+	{"nucleus0",  0,   kTUNE_NUCLEUS0},
+	{"nucleus",   0,   kTUNE_NUCLEUS1},
+	{"split",     0,   kTUNE_SPLIT},
+	{NULL, 0, -1}
+};
+
 static keywtab_t keywords[] = {
 	{"vowel",     tPHONEME_TYPE, phVOWEL},
 	{"liquid",    tPHONEME_TYPE, phLIQUID},
@@ -207,6 +250,7 @@ static keywtab_t keywords[] = {
 	{"procedure",  tSTATEMENT, kPROCEDURE},
 	{"endphoneme", tSTATEMENT, kENDPHONEME},
 	{"endprocedure", tSTATEMENT, kENDPROCEDURE},
+	{"equivalents", tSTATEMENT, kEQUIVALENTS},
 	{"import_phoneme",  tSTATEMENT, kIMPORT_PH},
 	{"stress_type",tSTATEMENT, kSTRESSTYPE},
 	{"starttype",  tSTATEMENT, kSTARTTYPE},
@@ -256,6 +300,7 @@ static keywtab_t keywords[] = {
 	{"LengthAdd",  tINSTRN1, i_ADD_LENGTH},
 	{"Lengthmod",  tINSTRN1, i_LENGTH_MOD},
 	{"lengthmod",  tINSTRN1, i_LENGTH_MOD},
+	{"ipa",        tINSTRN1, i_IPA_NAME},
 
 	// flags
 	{"wavef",      tPHONEME_FLAG, phWAVE},
@@ -271,6 +316,12 @@ static keywtab_t keywords[] = {
 	{"rhotic",     tPHONEME_FLAG, phRHOTIC},
 	{"nonsyllabic",tPHONEME_FLAG, phNONSYLLABIC},
 	{"lengthenstop",tPHONEME_FLAG, phLENGTHENSTOP},
+	{"nopause",    tPHONEME_FLAG, phNOPAUSE},
+	{"prevoice",   tPHONEME_FLAG, phPREVOICE},
+
+	{"flag1",      tPHONEME_FLAG, phFLAG1},
+	{"flag2",      tPHONEME_FLAG, phFLAG2},
+	{"flag3",      tPHONEME_FLAG, phFLAG3},
 
 	// voiced / unvoiced
 	{"vcd",	  tPHONEME_FLAG, phVOICED},
@@ -301,7 +352,7 @@ static keywtab_t keywords[] = {
 	{"glstop", tTRANSITION, 8},
 	{"lenadd", tTRANSITION, 9},
 	{"f4",     tTRANSITION, 10},
-	{"paus",   tTRANSITION, 11},
+	{"gpaus",  tTRANSITION, 11},
 	{"colr=",  tTRANSITION, 12},
 	{"amp=",   tTRANSITION, 13},  // set rms of 1st frame as fraction of rms of 2nd frame  (1/30ths)
 
@@ -310,7 +361,7 @@ static keywtab_t keywords[] = {
 
 
 static keywtab_t *keyword_tabs[] = {
-keywords, k_conditions, k_properties };
+keywords, k_conditions, k_properties, k_intonation };
 
 
 static PHONEME_TAB *phoneme_out;
@@ -318,14 +369,22 @@ static PHONEME_TAB *phoneme_out;
 static int n_phcodes_list[N_PHONEME_TABS];
 static PHONEME_TAB_LIST phoneme_tab_list2[N_PHONEME_TABS];
 static PHONEME_TAB *phoneme_tab2;
+static int phoneme_flags;
+static int place_articulation;
+
+static char *p_equivalence;
+static char equivalence_buf[20000];
+
 
 #define N_PROCS 50
 int n_procs;
 int  proc_addr[N_PROCS];
 char proc_names[40][N_PROCS];
 
+#define MAX_PROG_BUF 2000
 USHORT *prog_out;
-USHORT prog_buf[2000];
+USHORT *prog_out_max;
+USHORT prog_buf[MAX_PROG_BUF+20];
 
 
 static const char *KeyToMnem(keywtab_t *ktab, int type, int value)
@@ -353,6 +412,7 @@ static void DecompilePhoneme(FILE *f_out, PHONEME_TAB *ph, int compile_phoneme)
 	int ix;
 	int any;
 	const char *name;
+	char buf[120];
 
 	static const char *INV = "Invalid";
 
@@ -417,6 +477,18 @@ return;
 				fprintf(f_out,"%s",instn0_string[data1]);
 			}
 			else
+			if(type2 == i_IPA_NAME)
+			{
+				for(ix = 0; ix < data1; ix += 2)
+				{
+					instn = *pc++;
+					buf[ix] = instn >> 8;
+					buf[ix+1] = instn & 0xff;
+				}
+				buf[ix] = 0;
+				fprintf(f_out,"ipa %s",buf);
+			}
+			else
 			{
 				fprintf(f_out,"%s(",KeyToMnem(keywords, tINSTRN1, type2));
 				switch(instn1_paramtype[type2])
@@ -554,6 +626,7 @@ static FILE *f_phindex;
 static FILE *f_phtab;
 static FILE *f_phcontents;
 static FILE *f_errors = stderr;
+static FILE *f_prog_log = NULL;
 
 
 static FILE *f_in;
@@ -585,7 +658,6 @@ static REF_HASH_TAB *ref_hash_tab[256];
 
 
 #define N_ENVELOPES  30
-#define ENV_LEN  128
 static int n_envelopes = 0;
 static char envelope_paths[N_ENVELOPES][80];
 static unsigned char envelope_dat[N_ENVELOPES][ENV_LEN];
@@ -621,6 +693,7 @@ enum {
 	tKEYWORD,
 	tCONDITION,
 	tPROPERTIES,
+	tINTONATION,
 };
 
 int item_type;
@@ -783,16 +856,41 @@ static int ph_sorter(char **a, char **b)
 
 
 
-static void PrintPhonemesUsed(FILE *f, const char *dictname)
-{//=========================================================
+static void PrintPhonemesUsed(FILE *f, const char *dsource, const char *dictname)
+{//==============================================================================
 	int ix;
 	PHONEME_TAB *ph;
 	PHONEME_TAB *ph_tab[N_PHONEME_TAB];
 	int count = 0;
 	int n_ph = 0;
 	int section = 0;
+	time_t mod_time;
+	char fname[sizeof(path_home)+45];
+	struct stat statbuf;
+	char time_string[20];
+	const char *files[] = {"rules","list","listx"};
 
-	fprintf(f,"\n\nDictionary %s_dict\n",dictname);
+	// find the date-stamp of the dictionary source files
+	mod_time = 0;
+	for(ix=0; ix<3; ix++)
+	{
+		sprintf(fname,"%s%s_%s",dsource, dictname, files[ix]);
+		if(stat(fname,&statbuf) == 0)
+		{
+			if(statbuf.st_mtime > mod_time)
+				mod_time = statbuf.st_mtime;
+		}
+	}
+
+	if(mod_time > 0)
+	{
+		strftime(time_string, sizeof(time_string), "%F", localtime(&mod_time));
+		fprintf(f,"\n\nDictionary %s_dict  %s\n",dictname, time_string);
+	}
+	else
+	{
+		fprintf(f,"\n\nDictionary %s_dict\n",dictname);
+	}
 	fflush(f);
 
 	for(ix=0; (ix<N_PHONEME_TAB) && (phoneme_tab[ix] != NULL); ix++)
@@ -948,7 +1046,7 @@ static wxString CompileAllDictionaries()
 		{
 			memset(phoneme_tab_flags,0,sizeof(phoneme_tab_flags));
 			FindPhonemesUsed();
-			PrintPhonemesUsed(f_phused,dictname);
+			PrintPhonemesUsed(f_phused, path_dsource, dictname);
 		}
 
 		cont = dir.GetNext(&filename);
@@ -974,6 +1072,8 @@ static wxString CompileAllDictionaries()
 
 static void error(const char *format, const char *string)
 {//======================================================
+	if(string==NULL)
+		string = "";
 	fprintf(f_errors,"%4d:  ",linenum-1);
 	fprintf(f_errors,format,string);
 	fprintf(f_errors,"\n");
@@ -982,7 +1082,7 @@ static void error(const char *format, const char *string)
 
 static void Error(const char *string)
 {//==================================
-	error(string,NULL);
+	error("%s",string);
 }
 
 static FILE *fopen_log(FILE *f_log, const char *fname,const char *access)
@@ -1018,7 +1118,7 @@ static unsigned int StringToWord(const char *string)
 
 
 static MNEM_TAB reserved_phonemes[] = {
-	{"<",  phonCONTROL},      // NOT USED
+	{"_\001",  phonCONTROL},      // NOT USED
 	{"%",  phonSTRESS_U},
 	{"%%", phonSTRESS_D},
 	{",",  phonSTRESS_2},
@@ -1149,6 +1249,15 @@ static void unget_char(unsigned int c)
 }
 
 
+int CheckNextChar()
+{//================
+	int c;
+	while(((c = get_char()) == ' ') || (c == '\t'));
+	unget_char(c);
+	return(c);
+}  // end of CheckNextChar
+
+
 static int NextItem(int type)
 {//==========================
 	int  acc;
@@ -1156,7 +1265,6 @@ static int NextItem(int type)
 	unsigned char c2;
 	int  ix;
 	int  sign;
-	int  keyword;
 	char *p;
 	keywtab_t *pk;
 	
@@ -1224,8 +1332,6 @@ static int NextItem(int type)
 	if(!feof(f_in))
 		unget_char(c);
 
-	keyword = -1;
-
 	if(type == tSTRING)
 	{
 		return(0);
@@ -1243,7 +1349,12 @@ static int NextItem(int type)
 			p++;
 		}
 		if(!isdigit(*p))
-			error("Expected a number",NULL);
+		{
+			if((type == tNUMBER) && (*p == '-'))
+				error("Expected an unsigned number",NULL);
+			else
+				error("Expected a number",NULL);
+		}
 		while(isdigit(*p))
 		{
 			acc *= 10;
@@ -1254,7 +1365,7 @@ static int NextItem(int type)
 	}
 
 
-	if((type >= tKEYWORD) && (type <= tPROPERTIES))
+	if((type >= tKEYWORD) && (type <= tINTONATION))
 	{
 		pk = keyword_tabs[type-tKEYWORD];
 		while(pk->mnem != NULL)
@@ -1502,7 +1613,7 @@ int LoadSpect(const char *path, int control)
 	displ = ftell(f_phdata);
 
 	seq_out.n_frames=0;
-	seq_out.flags=0;
+	seq_out.sqflags=0;
 	seq_out.length_total=0;
 
 	total = 0;	
@@ -1649,7 +1760,7 @@ for(ix=0; ix<8; ix++)
 	if(klatt_flag)
 	{
 		seqk_out.n_frames = seq_out.n_frames;
-		seqk_out.flags = seq_out.flags;
+		seqk_out.sqflags = seq_out.sqflags;
 		seqk_out.length_total = seq_out.length_total;
 
 		ix = (char *)(&seqk_out.frame[seqk_out.n_frames]) - (char *)(&seqk_out);
@@ -1690,12 +1801,12 @@ static int LoadWavefile(FILE *f, const char *fname)
 	sr2 = Read4Bytes(f);
 	fseek(f,40,SEEK_SET);	
 
-	if((sr1 != samplerate) || (sr2 != sr1*2))
+	if((sr1 != samplerate_native) || (sr2 != sr1*2))
 	{
 #ifdef PLATFORM_WINDOWS
-		if(sr1 != samplerate)
+		if(sr1 != samplerate_native)
 		{
-			fprintf(f_errors,"Wrong samplerate %d, wants %d\n",sr1,samplerate);
+			fprintf(f_errors,"Wrong samplerate %d, wants %d\n",sr1,samplerate_native);
 			error("Wrong samplerate: %s",fname);
 		}
 		if(sr2 != sr1*2)
@@ -1710,7 +1821,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 			if((fd_temp = mkstemp(fname_temp)) >= 0)
 			{
 				close(fd_temp);
-				sprintf(command,"sox \"%s%s.wav\" -r %d -c 1 -w  %s polyphase\n",path_source,fname,samplerate,fname_temp);
+				sprintf(command,"sox \"%s%s.wav\" -r %d -c 1 -w  %s polyphase\n",path_source,fname,samplerate_native,fname_temp);
 				if(system(command) < 0)
 				{
 					error("Failed to resample: %s",command);
@@ -1822,8 +1933,11 @@ static int LoadEnvelope(FILE *f, const char *fname)
 	displ = ftell(f_phdata);
 
 	fseek(f,12,SEEK_SET);
-	fread(buf,1,128,f);
-	fwrite(buf,1,128,f_phdata);
+	if(fread(buf,128,1,f) == 0)
+	{
+		error("Failed to read envelope: %s",fname);
+	}
+	fwrite(buf,128,1,f_phdata);
 
 	if(n_envelopes < N_ENVELOPES)
 	{
@@ -2091,7 +2205,29 @@ int CompileSound(int keyword, int isvowel)
 	strcpy(path, item_string);
 	if(item_terminator == ',')
 	{
-		value = NextItemBrackets(tSIGNEDNUMBER,1);
+		if((keyword == kVOWELSTART) || (keyword == kVOWELENDING))
+		{
+			value = NextItemBrackets(tSIGNEDNUMBER,1);
+			if(value > 127)
+			{
+				value = 127;
+				error("Parameter > 127",NULL);
+			}
+			if(value < -128)
+			{
+				value = -128;
+				error("Parameter < -128",NULL);
+			} 
+		}
+		else
+		{
+			value = NextItemBrackets(tNUMBER,1);
+			if(value > 255)
+			{
+				value = 255;
+				error("Parameter > 255",NULL);
+			}
+		}
 	}
 	addr = LoadDataFile(path, isvowel);
 	addr = addr / 4;   // addr is words not bytes
@@ -2130,6 +2266,12 @@ int CompileIf(int elif)
 
 	while(!finish)
 	{
+		if(prog_out >= prog_out_max)
+		{
+			error("Phoneme program too large", NULL);
+			return(0);
+		}
+
 		if((key = NextItem(tCONDITION)) < 0)
 			error("Expected a condition, not '%s'",item_string);
 	
@@ -2360,6 +2502,23 @@ static int CompileSwitch(int type)
 
 
 
+static PHONEME_TAB_LIST *FindPhonemeTable(const char *string)
+{//==========================================================
+	int ix;
+
+	for(ix=0; ix<n_phoneme_tabs; ix++)
+	{
+		if(strcmp(phoneme_tab_list2[ix].name, string) == 0)
+		{
+			return(&phoneme_tab_list2[ix]);
+		}
+	}
+	error("Unknown phoneme table: '%s'",string);
+	return(NULL);
+}  // end of FindPhonemeTable
+
+
+
 static PHONEME_TAB *FindPhoneme(const char *string)
 {//================================================
 	PHONEME_TAB_LIST *phtab = NULL;
@@ -2368,7 +2527,7 @@ static PHONEME_TAB *FindPhoneme(const char *string)
 	char *phname;
 	char buf[200];
 
-	// is this the name of a phoneme which is ijn scope
+	// is this the name of a phoneme which is in scope
 	if((strlen(string) <= 4) && ((ix = LookupPhoneme(string,0)) != -1))
 	{
 		return(&phoneme_tab2[ix]);
@@ -2381,17 +2540,9 @@ static PHONEME_TAB *FindPhoneme(const char *string)
 		*phname++ = 0;
 	}
 
-	for(ix=0; ix<n_phoneme_tabs; ix++)
-	{
-		if(strcmp(phoneme_tab_list2[ix].name,buf) == 0)
-		{
-			phtab = &phoneme_tab_list2[ix];
-			break;
-		}
-	}
+	phtab = FindPhonemeTable(buf);
 	if(phtab == NULL)
 	{
-		error("Unknown phoneme table: '%s'",buf);
 		return(NULL);  // phoneme table not found
 	}
 
@@ -2456,6 +2607,18 @@ static void CallPhoneme(void)
 		if((ph = FindPhoneme(item_string)) == NULL)
 			return;
 		addr = ph->program;
+
+		if(phoneme_out->type == phINVALID)
+		{
+			// Phoneme type has not been set. Copy it from the called phoneme
+			phoneme_out->type = ph->type;
+			phoneme_flags = ph->phflags & ~phARTICULATION;
+			place_articulation = (ph->phflags & phARTICULATION) >> 16;
+			phoneme_out->start_type = ph->start_type;
+			phoneme_out->end_type = ph->end_type;
+			phoneme_out->std_length = ph->std_length;
+			phoneme_out->length_mod = ph->length_mod;
+		}
 	}
 
 	*prog_out++ = i_CALLPH + (addr >> 16);
@@ -2482,17 +2645,24 @@ static void InstnPlusPhoneme(int instn)
 int CompilePhoneme(int compile_phoneme)
 {//====================================
 	int endphoneme = 0;
-	int phoneme_flags = 0;
-	int place_articulation = 0;
 	int keyword;
 	int value;
 	int phcode = 0;
+	int ix;
+	unsigned int c;
+	char *p;
+	char number_buf[12];
+	char ipa_buf[N_ITEM_STRING];
 	PHONEME_TAB phoneme_out2;
+	PHONEME_PROG_LOG phoneme_prog_log;
 
 	prog_out = prog_buf;
+	prog_out_max = &prog_buf[MAX_PROG_BUF-1];
 	if_level = 0;
 	if_stack[0].returned = 0;
 	after_if = 0;
+	phoneme_flags = 0;
+	place_articulation = 0;
 
 	NextItem(tSTRING); 
 	if(compile_phoneme)
@@ -2602,6 +2772,42 @@ int CompilePhoneme(int compile_phoneme)
 				value = NextItem(tNUMBER);
 				phoneme_out->length_mod = value;
 				break;
+
+			case i_IPA_NAME:
+				NextItem(tSTRING);
+
+				if(strcmp(item_string,"NULL")==0)
+					strcpy(item_string," ");
+
+				// copy the string, recognize characters in the form U+9999
+				p = item_string;
+				ix = 0;
+				while((c = *p++) != 0)
+				{
+					if((c=='U') && (p[0]=='+'))
+					{
+						// U+9999
+						memcpy(number_buf,&p[1],4);  // U+ should be followed by 4 hex digits
+						number_buf[4] = 0;
+						c = 0;
+						sscanf(number_buf,"%x",&c);
+						p += 5;
+						ix += utf8_out(c, &ipa_buf[ix]);
+					}
+					else
+					{
+						ipa_buf[ix++] = c;
+					}
+				}
+				ipa_buf[ix] = 0;
+				value = strlen(ipa_buf);   // number of UTF-8 bytes
+
+				*prog_out++ = (i_IPA_NAME << 8) + value;
+				for(ix=0; ix < value; ix += 2)
+				{
+					*prog_out++ = (ipa_buf[ix] << 8) + (ipa_buf[ix+1] & 0xff);
+				}
+				break;
 			}
 			break;
 
@@ -2647,6 +2853,11 @@ int CompilePhoneme(int compile_phoneme)
 			case kSTRESSTYPE:
 				value = NextItem(tNUMBER);
 				phoneme_out->std_length = value;
+				if(prog_out > prog_buf)
+				{
+					error("stress phonemes can't contain program instructions",NULL);
+					prog_out = prog_buf;
+				}
 				break;
 
 			case kIF:
@@ -2681,6 +2892,7 @@ int CompilePhoneme(int compile_phoneme)
 				break;
 
 			case kFMT:
+				if_stack[if_level].returned = 1;
 				DecThenCount();
 				if(phoneme_out->type == phVOWEL)
 					endphoneme = CompileSound(keyword,1);
@@ -2787,6 +2999,14 @@ int CompilePhoneme(int compile_phoneme)
 		// write out the program for this phoneme
 		fflush(f_phindex);
 		phoneme_out->program = ftell(f_phindex) / sizeof(USHORT);
+
+		if(f_prog_log != NULL)
+		{
+			phoneme_prog_log.addr = phoneme_out->program;
+			phoneme_prog_log.length = prog_out - prog_buf;
+			fwrite(&phoneme_prog_log, 1, sizeof(phoneme_prog_log), f_prog_log);
+		}
+
 		if(compile_phoneme == 0)
 		{
 			proc_addr[n_procs++] =  ftell(f_phindex) / sizeof(USHORT);
@@ -2838,6 +3058,7 @@ static void WritePhonemeTables()
 		fputc(phoneme_tab_list2[ix].includes,f_phtab);
 		fputc(0,f_phtab);
 		fputc(0,f_phtab);
+		Write4Bytes(f_phtab, phoneme_tab_list2[ix].equivalence_tables);   // byte index into phondata for equivalence tables
 
 		fwrite(phoneme_tab_list2[ix].name,1,N_PHONEME_TAB_NAME,f_phtab);
 
@@ -2860,6 +3081,8 @@ static void WritePhonemeTables()
 static void EndPhonemeTable()
 {//==========================
 	int  ix;
+	int *pw;
+	int length;
 
 	if(n_phoneme_tabs == 0)
 		return;
@@ -2874,10 +3097,24 @@ static void EndPhonemeTable()
 			fprintf(f_errors,"%3d: Phoneme [%s] not declared, referenced at line %d\n",linenum,
 				WordToString(phoneme_tab2[ix].mnemonic),int(phoneme_tab2[ix].program));
 			error_count++;
+			phoneme_tab2[ix].type = 0;   // prevent the error message repeating
 		}
 	}
 
 	n_phcodes_list[n_phoneme_tabs-1] = n_phcodes;
+
+	if((length = p_equivalence - equivalence_buf) > 0)
+	{
+		// terminate the list of phoneme equivalence tables
+		pw = (int *)p_equivalence;
+		pw[0] = 0;
+
+		// write the equivalence data into phondata, and remember it's address
+		ix = ftell(f_phdata);
+		fprintf(f_phcontents,"Q  0x%.5x  %s\n", ix, phoneme_tab_list2[n_phoneme_tabs-1].name);
+		phoneme_tab_list2[n_phoneme_tabs-1].equivalence_tables = ix;
+		fwrite(equivalence_buf, length+4, 1, f_phdata);
+	}
 }
 
 
@@ -2905,11 +3142,12 @@ static void StartPhonemeTable(const char *name)
 	if(gui_flag)
 		progress->Update(n_phoneme_tabs);
 
+	memset(&phoneme_tab_list2[n_phoneme_tabs], 0, sizeof(PHONEME_TAB_LIST));
 	phoneme_tab_list2[n_phoneme_tabs].phoneme_tab_ptr = phoneme_tab2 = p;
-	memset(phoneme_tab_list2[n_phoneme_tabs].name, 0, N_PHONEME_TAB_NAME);
 	strncpy0(phoneme_tab_list2[n_phoneme_tabs].name, name, N_PHONEME_TAB_NAME);
 	n_phcodes = 1;
 	phoneme_tab_list2[n_phoneme_tabs].includes = 0;
+	p_equivalence = equivalence_buf;
 
 	if(n_phoneme_tabs > 0)
 	{
@@ -2944,6 +3182,117 @@ static void StartPhonemeTable(const char *name)
 }  // end of StartPhonemeTable
 
 
+static void CompileEquivalents()
+{//=============================
+	// a list of phonemes in another language and the equivalent phoneme strings in this language
+
+	int ix;
+	int n_names;
+	int n_bytes;
+	int foreign_phoneme;
+	int foreign_error = 0;
+	int remove_stress = 0;
+	char *p_start;
+	char *p;
+	int foreign_table;
+	char foreign_table_name[40];
+	char line_buf[80];
+	char names[6][80];
+	char phcode[7];
+	char save_voice_name[80];
+
+	NextItem(tSTRING);
+	strcpy(foreign_table_name, item_string);
+	strcpy(save_voice_name,voice_name2);
+
+	if((foreign_table = SelectPhonemeTableName(foreign_table_name)) < 0)
+	{
+		if(strcmp(foreign_table_name, "NULL") != 0)
+		{
+			error("Unknown phoneme table '%s'", foreign_table_name);
+		}
+		foreign_error = 1;
+		foreign_phoneme = 0;
+	}
+
+	p_start = p_equivalence;
+	p_equivalence += 8;
+
+	p_start[0] = foreign_table;
+
+	linenum--;
+	while(!feof(f_in))
+	{
+		linenum++;
+		if(fgets(line_buf, sizeof(line_buf), f_in) == NULL)
+			break;
+
+		if((p = strstr(line_buf,"//")) != NULL)
+			*p = 0;
+
+		for(ix=0; ix<6; ix++)
+			names[ix][0] = 0;
+		n_names = sscanf(line_buf,"%s %s %s %s %s %s",names[0],names[1],names[2],names[3],names[4],names[5]);
+		if(n_names < 1)
+			continue;
+
+		if(strcmp(names[0],"endphoneme") == 0)
+			break;
+
+		if(foreign_error)
+			continue;
+
+		if(strcmp(names[0],"remove_stress") == 0)
+		{
+			remove_stress = 1;
+			continue;
+		}
+
+		if(p_equivalence > &equivalence_buf[sizeof(equivalence_buf) - 16])
+		{
+			error("'equivalents' tables are too large",NULL);
+			break;
+		}
+
+		if(foreign_error == 0)
+		{
+			phcode[0] = foreign_phoneme = LookupPhonemeString(names[0]);
+			if(foreign_phoneme == 0)
+			{
+				sprintf(line_buf,"%s/%s", foreign_table_name, names[0]);
+				error("Unknown phoneme '%s'", line_buf);
+			}
+		}
+
+		for(ix=1; ix<n_names; ix++)
+		{
+			phcode[ix] = LookupPhoneme(names[ix], 1);
+		}
+
+		// only write a translation if it has an effect
+		if((n_names > 2) || (phcode[0] != phcode[1]))
+		{
+			// write: foreign phoneme number, then a string of local phoneme numbers
+			memcpy(p_equivalence, phcode, n_names);
+			p_equivalence += n_names;
+			*p_equivalence++ = 0;
+		}
+	}
+	*p_equivalence++ = 0;
+
+	p_equivalence = (char *)((long int)(p_equivalence + 3) & ~0x3);  // align to word boundary
+	n_bytes = p_equivalence - p_start;
+	p_start[1] = remove_stress;
+	n_bytes = n_bytes / 4;
+	p_start[2] = n_bytes >> 8;    // index of next table
+	p_start[3] = n_bytes;
+
+	LoadVoice(voice_name2,0);  // reset the original phoneme table
+	LoadVoiceVariant(save_voice_name,0);
+}  // end of CompileEquivalents
+
+
+
 static void CompilePhonemeFiles()
 {//==============================
 	int item;
@@ -2974,8 +3323,9 @@ static void CompilePhonemeFiles()
 
 		item = NextItem(tKEYWORD);
 
-		if(item == kINCLUDE)
+		switch(item)
 		{
+		case kINCLUDE:
 			NextItem(tSTRING);
 			sprintf(buf,"%s%s",path_source,item_string);
 			if((stack_ix < N_STACK) && (f = fopen_log(f_errors,buf,"rb")) != NULL)
@@ -2989,37 +3339,42 @@ static void CompilePhonemeFiles()
 				strncpy0(current_fname,item_string,sizeof(current_fname));
 				linenum = 1;
 			}
-		}
-		else
-		if(item == kPHONEMETABLE)
-		{
+			break;
+
+		case kPHONEMETABLE:
 			EndPhonemeTable();
 			NextItem(tSTRING);	// name of the new phoneme table
 			StartPhonemeTable(item_string);
-		}
-		else
-		if(item == kPHONEMESTART)
-		{
+			break;
+
+		case kPHONEMESTART:
 			if(n_phoneme_tabs == 0)
 			{
 				Error("phonemetable is missing");
 				return;
 			}
 			CompilePhoneme(1);
-		}
-		else
-		if(item == kPROCEDURE)
-		{
+			break;
+
+		case kPROCEDURE:
 			CompilePhoneme(0);
+			break;
+
+		case kEQUIVALENTS:
+			CompileEquivalents();
+			break;
+
+		default:
+			if(!feof(f_in))
+				Error("Keyword 'phoneme' expected");
+			break;
 		}
-		else
-		if(!feof(f_in))
-			Error("Keyword 'phoneme' expected");
 	}
 	phoneme_tab2[n_phcodes+1].mnemonic = 0;  // terminator
 }  //  end of CompilePhonemeFiles
 
 
+
 static void CompilePhonemeData2(const char *source)
 {//================================================
 	char fname[sizeof(path_source)+40];
@@ -3087,6 +3442,7 @@ memset(markers_used,0,sizeof(markers_used));
 "#   S - A SPECT_SEQ structure\n"
 "#   W - A wavefile segment\n"
 "#   E - An envelope\n"
+"#   Q - Phoneme equivalence tables\n"
 "#\n"
 "# Address is the displacement within phondata of this item\n"
 "#\n"
@@ -3108,9 +3464,12 @@ memset(markers_used,0,sizeof(markers_used));
 		return;
 	}
 
+	sprintf(fname,"%scompile_prog_log",path_source);
+	f_prog_log = fopen_log(f_errors,fname,"wb");
+
 	if(gui_flag)
 	{
-		progress = new wxProgressDialog(_T("Phonemes"),_T(""),progress_max);
+		progress = new wxProgressDialog(_T("Compiling"),_T(""),progress_max);
 	}
 	else
 	{
@@ -3135,6 +3494,7 @@ fprintf(f_errors,"\nRefs %d,  Reused %d\n",count_references,duplicate_references
 	fclose(f_in);
 	fclose(f_phdata);
 	fclose(f_errors);
+	fclose(f_prog_log);
 	fclose(f_phindex);
 	fclose(f_phtab);
 	fclose(f_phcontents);
@@ -3144,9 +3504,9 @@ fprintf(f_errors,"\nRefs %d,  Reused %d\n",count_references,duplicate_references
 
 	CompileReport();
 	report_dict = CompileAllDictionaries();
-#ifdef deleted
+//#ifdef deleted
 	DrawEnvelopes();
-#endif
+//#endif
 
 	if(gui_flag)
 	{
@@ -3240,7 +3600,7 @@ void CompileMbrola()
 	wxFileName filename = wxFileName(filepath);
 	strcpy(mbrola_voice,filename.GetName().mb_str(wxConvLocal));
 	sprintf(buf,"%s/mbrola_ph/%s_phtrans",path_home,mbrola_voice);
-	if((f_out = fopen(buf,"w")) == NULL)
+	if((f_out = fopen(buf,"wb")) == NULL)
 	{
 		wxLogError(_T("Can't write to: ")+wxString(buf,wxConvLocal));
 		return;
@@ -3260,6 +3620,360 @@ void CompileMbrola()
 
 
 
+static const char *preset_tune_names[] = {
+"s1", "c1", "q1", "e1", NULL};
+
+static const TUNE default_tune = {
+{0,0,0,0,0,0,0,0,0,0,0,0},
+{0,0,0,0},
+{0, 40, 24, 8, 0, 0, 0, 0},
+46, 57, PITCHfall, 16, 0, 0,
+255, 78, 50, 255,
+3, 5,
+{-7,-7,-7}, {-7,-7,-7},
+PITCHfall, 64, 8,
+PITCHfall, 70, 18, 24, 12,
+PITCHfall, 70, 18, 24, 12, 0,
+{0,0,0,0,0,0,0,0}, 0
+};
+
+#define N_TUNE_NAMES  100
+
+MNEM_TAB envelope_names[] = {
+	{"fall", 0},
+	{"rise", 2},
+	{"fall-rise", 4},
+	{"fall-rise2", 6},
+	{"rise-fall",  8},
+	{"fall-rise3",10},
+	{"fall-rise4",12},
+	{"fall2", 14},
+	{"rise2", 16},
+	{"rise-fall-rise", 18},
+	{NULL, -1} };
+
+
+//	env_fallrise3, env_fallrise3,
+//	env_fallrise4, env_fallrise4,
+//	env_fall2, env_fall2,
+//	env_rise2, env_rise2,
+//	env_risefallrise, env_risefallrise
+
+int LookupEnvelope(const char *name)
+{//=================================
+	return(LookupMnem(envelope_names, name));
+}
+
+
+void CompileIntonation()
+{//=====================
+	int ix;
+	char *p;
+	char c;
+	int keyword;
+	int compiling_tune = 0;
+	int n_tune_names = 0;
+	int done_split;
+	int done_onset = 0;
+	int done_last = 0;
+	int n_preset_tunes = 0;
+	int found;
+	int tune_number = 0;
+	FILE *f_out;
+	wxString report;
+	TUNE *tune_data;
+	TUNE new_tune;
+
+
+	char name[12];
+	char fname_errors[sizeof(path_source)+120];
+	char tune_names[N_TUNE_NAMES][12];
+	char buf[sizeof(path_source)+120];
+
+	error_count = 0;
+
+	sprintf(fname_errors,"%s%s",path_source,"error_intonation");
+	if((f_errors = fopen(fname_errors,"w")) == NULL)
+		f_errors = stderr;
+
+	sprintf(buf,"%sintonation.txt",path_source);
+	if((f_in = fopen(buf, "r")) == NULL)
+	{
+		sprintf(buf,"%sintonation",path_source);
+		if((f_in = fopen_log(f_errors, buf, "r")) == NULL)
+		{
+			wxLogError(_T("Can't read file: ") + wxString(buf,wxConvLocal));
+			fclose(f_errors);
+			return;
+		}
+	}
+
+
+	for(ix=0; preset_tune_names[ix] != NULL; ix++)
+	{
+		strcpy(tune_names[ix], preset_tune_names[ix]);
+	}
+	n_tune_names = ix;
+	n_preset_tunes = ix;
+
+	// make a list of the tune names
+	while(!feof(f_in))
+	{
+		if(fgets(buf,sizeof(buf),f_in) == NULL)
+			break;
+
+		if((memcmp(buf,"tune",4)==0) && isspace(buf[4]))
+		{
+			p = &buf[5];
+			while(isspace(*p)) p++;
+
+			ix = 0;
+			while((ix < int(sizeof(name) - 1)) && !isspace(*p))
+			{
+				name[ix++] = *p++;
+			}
+			name[ix] = 0;
+
+			found = 0;
+			for(ix = 0; ix < n_tune_names; ix++)
+			{
+				if(strcmp(name, tune_names[ix]) == 0)
+				{
+					found = 1;
+					break;
+				}
+			}
+
+			if(found == 0)
+			{
+				strncpy0(tune_names[n_tune_names++], name, sizeof(name));
+
+				if(n_tune_names >= N_TUNE_NAMES)
+					break;
+			}
+		}
+	}
+	rewind(f_in);
+	linenum = 1;
+
+	tune_data = (TUNE *)calloc(sizeof(TUNE), n_tune_names);
+	if(tune_data == NULL)
+	{
+		fprintf(f_errors, "Failed to allocate data for tunes\n");
+		fclose(f_in);
+		fclose(f_errors);
+		return;
+	}
+
+	sprintf(buf,"%s/intonations",path_home);
+	f_out = fopen_log(f_errors, buf, "wb");
+	if(f_out == NULL)
+	{
+		fclose(f_in);
+		fclose(f_errors);
+		return;
+	}
+
+	while(!feof(f_in))
+	{
+		keyword = NextItem(tINTONATION);
+
+		switch(keyword)
+		{
+		case kTUNE:
+			if(compiling_tune)
+			{
+			}
+			compiling_tune = 1;
+			done_split = 0;
+
+			memcpy(&new_tune, &default_tune, sizeof(TUNE));
+			NextItem(tSTRING);
+			strncpy0(new_tune.name, item_string, sizeof(new_tune.name));
+
+			found = 0;
+			tune_number = 0;
+			for(ix=0; ix < n_tune_names; ix++)
+			{
+				if(strcmp(new_tune.name, tune_names[ix]) == 0)
+				{
+					found = 1;
+					tune_number = ix;
+
+					if(tune_data[ix].name[0] != 0)
+						found = 2;
+					break;
+				}
+			}
+//			fprintf(f_errors,"tune %s  (%d)\n", new_tune.name, tune_number);
+			if(found == 2)
+			{
+				error("Duplicate tune name: '%s'",new_tune.name);
+			}
+			if(found == 0)
+			{
+				error("Bad tune name: '%s;", new_tune.name);
+			}
+			break;
+
+		case kENDTUNE:
+			compiling_tune = 0;
+
+			if(done_onset == 0)
+			{
+				new_tune.unstr_start[0] = new_tune.unstr_start[1];
+				new_tune.unstr_end[0] = new_tune.unstr_end[1];
+			}
+			if(done_last == 0)
+			{
+				new_tune.unstr_start[2] = new_tune.unstr_start[1];
+				new_tune.unstr_end[2] = new_tune.unstr_end[1];
+			}
+			memcpy(&tune_data[tune_number], &new_tune, sizeof(TUNE));
+			break;
+
+		case kTUNE_PREHEAD:
+			new_tune.prehead_start = NextItem(tNUMBER);
+			new_tune.prehead_end = NextItem(tNUMBER);
+			break;
+
+		case kTUNE_ONSET:
+			new_tune.onset = NextItem(tNUMBER);
+			new_tune.unstr_start[0] = NextItem(tSIGNEDNUMBER);
+			new_tune.unstr_end[0] = NextItem(tSIGNEDNUMBER);
+			done_onset = 1;
+			break;
+
+		case kTUNE_HEADLAST:
+			new_tune.head_last = NextItem(tNUMBER);
+			new_tune.unstr_start[2] = NextItem(tSIGNEDNUMBER);
+			new_tune.unstr_end[2] = NextItem(tSIGNEDNUMBER);
+			done_last = 1;
+			break;
+
+		case kTUNE_HEADENV:
+			NextItem(tSTRING);
+			if((ix = LookupEnvelope(item_string)) < 0)
+				error("Bad envelope name: '%s'",item_string);
+			else
+				new_tune.stressed_env = ix;
+			new_tune.stressed_drop = NextItem(tNUMBER);
+			break;
+
+		case kTUNE_HEAD:
+			new_tune.head_max_steps = NextItem(tNUMBER);
+			new_tune.head_start = NextItem(tNUMBER);
+			new_tune.head_end = NextItem(tNUMBER);
+			new_tune.unstr_start[1] = NextItem(tSIGNEDNUMBER);
+			new_tune.unstr_end[1] = NextItem(tSIGNEDNUMBER);
+			break;
+
+		case kTUNE_HEADEXTEND:
+			// up to 8 numbers
+			for(ix=0; ix < int(sizeof(new_tune.head_extend)); ix++)
+			{
+				if(!isdigit(c = CheckNextChar()) && (c != '-'))
+					break;
+
+				new_tune.head_extend[ix] = (NextItem(tSIGNEDNUMBER) * 64) / 100;  // convert from percentage to 64ths
+			}
+			new_tune.n_head_extend = ix;    // number of values
+			break;
+
+		case kTUNE_NUCLEUS0:
+			NextItem(tSTRING);
+			if((ix = LookupEnvelope(item_string)) < 0)
+			{
+				error("Bad envelope name: '%s'",item_string);
+				break;
+			}
+			new_tune.nucleus0_env = ix;
+			new_tune.nucleus0_max = NextItem(tNUMBER);
+			new_tune.nucleus0_min = NextItem(tNUMBER);
+			break;
+
+		case kTUNE_NUCLEUS1:
+			NextItem(tSTRING);
+			if((ix = LookupEnvelope(item_string)) < 0)
+			{
+				error("Bad envelope name: '%s'",item_string);
+				break;
+			}
+			new_tune.nucleus1_env = ix;
+			new_tune.nucleus1_max = NextItem(tNUMBER);
+			new_tune.nucleus1_min = NextItem(tNUMBER);
+			new_tune.tail_start = NextItem(tNUMBER);
+			new_tune.tail_end = NextItem(tNUMBER);
+
+			if(!done_split)
+			{
+				// also this as the default setting for 'split'
+				new_tune.split_nucleus_env = ix;
+				new_tune.split_nucleus_max = new_tune.nucleus1_max;
+				new_tune.split_nucleus_min = new_tune.nucleus1_min;
+				new_tune.split_tail_start = new_tune.tail_start;
+				new_tune.split_tail_end = new_tune.tail_end;
+			}
+			break;
+
+		case kTUNE_SPLIT:
+			NextItem(tSTRING);
+			if((ix = LookupEnvelope(item_string)) < 0)
+			{
+				error("Bad envelope name: '%s'",item_string);
+				break;
+			}
+			done_split = 1;
+			new_tune.split_nucleus_env = ix;
+			new_tune.split_nucleus_max = NextItem(tNUMBER);
+			new_tune.split_nucleus_min = NextItem(tNUMBER);
+			new_tune.split_tail_start = NextItem(tNUMBER);
+			new_tune.split_tail_end = NextItem(tNUMBER);
+			NextItem(tSTRING);
+			item_string[12] = 0;
+			for(ix=0; ix < n_tune_names; ix++)
+			{
+				if(strcmp(item_string, tune_names[ix]) == 0)
+					break;
+			}
+
+			if(ix == n_tune_names)
+				error("Tune '%s' not found",item_string);
+			else
+				new_tune.split_tune = ix;
+			break;
+
+		default:
+			error("Unexpected: '%s'",item_string);
+			break;
+		}
+	}
+
+	for(ix = 0; ix < n_preset_tunes; ix++)
+	{
+		if(tune_data[ix].name[0] == 0)
+		{
+			error("Tune '%s' not defined", preset_tune_names[ix]);
+		}
+	}
+	fwrite(tune_data, n_tune_names, sizeof(TUNE), f_out);
+	fclose(f_in);
+	fclose(f_out);
+	fclose(f_errors);
+
+	report.Printf(_T("Compiled %d intonation tunes: %d errors."),n_tune_names, error_count);
+	wxLogStatus(report);
+
+	if(error_count > 0)
+	{
+		DisplayErrorFile(fname_errors);
+	}
+	LoadPhData();
+
+}   // end of CompileIntonation
+
+
+
 void CompilePhonemeData()
 {
 	CompilePhonemeData2("phonemes");
