@@ -65,6 +65,7 @@ extern void DictionarySort(const char *dictname);
 extern void init_z();
 extern void CompilePhonemeData(void);
 extern void CompileMbrola();
+extern void CompileIntonation();
 extern void InitSpectrumDisplay();
 extern void InitProsodyDisplay();
 extern void InitWaveDisplay();
@@ -88,6 +89,7 @@ wxNotebook *notebook = NULL;
 wxProgressDialog *progress;
 int progress_max;
 int gui_flag = 0;
+int frame_x, frame_y, frame_w, frame_h;
 
 wxList my_children;
 
@@ -127,8 +129,6 @@ char param[80];
 
 if(argc > 1)
 {
-extern void VoiceReset(int control);
-
 	p = argv[1];
 	j = 0;
 	while((param[j] = p[j]) != 0) j++;
@@ -148,10 +148,12 @@ extern void VoiceReset(int control);
 	if(strcmp(param,"--compile")==0)
 	{
 		CompilePhonemeData();
+		CompileIntonation();
 		exit(0);
 	}
 }
 
+	ConfigInit();
 	gui_flag = 1;
 	// It seems that the wctype functions don't work until the locale has been set
 	// to something other than the default "C".  Then, not only Latin1 but also the
@@ -162,20 +164,25 @@ extern void VoiceReset(int control);
 			setlocale(LC_CTYPE,"");
 	}
 
+	if((frame_w == 0) || (frame_h == 0))
+	{
+		frame_w = 1024;
+		frame_h = 768;
+	}
 
   // Create the main frame window
-	myframe = new MyFrame(NULL, -1, AppName, wxPoint(0, 0), wxSize(1024, 768),
+	myframe = new MyFrame(NULL, -1, AppName, wxPoint(frame_x, frame_y), wxSize(frame_w, frame_h),
                       wxDEFAULT_FRAME_STYLE |
                       wxNO_FULL_REPAINT_ON_RESIZE |
                       wxHSCROLL | wxVSCROLL);
 
 
 	// Make a menubar
-	myframe->SetMenuBar(MakeMenu(0));
+	myframe->SetMenuBar(MakeMenu(0, voice_name2));
 	myframe->CreateStatusBar();
 	myframe->SetVoiceTitle(voice_name2);
 
-	myframe->Maximize();
+//	myframe->Maximize();
 	myframe->Show(TRUE);
 
 	SetTopWindow(myframe);
@@ -191,7 +198,9 @@ BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
    EVT_MENU(MENU_SPECTRUM, MyFrame::OnNewWindow)
    EVT_MENU(MENU_SPECTRUM2, MyFrame::OnNewWindow)
    EVT_MENU(MENU_PROSODY, MyFrame::OnProsody)
-   EVT_MENU(MENU_PARAMS, MyFrame::OnOptions)
+   EVT_MENU(MENU_OPT_SPEED, MyFrame::OnOptions)
+   EVT_MENU(MENU_OPT_PUNCT, MyFrame::OnOptions)
+   EVT_MENU(MENU_OPT_SPELL, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH0, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH1, MyFrame::OnOptions)
    EVT_MENU(MENU_PATH2, MyFrame::OnOptions)
@@ -203,10 +212,12 @@ BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
 	EVT_MENU(MENU_FORMAT_DICTIONARY, MyFrame::OnTools)
 	EVT_MENU(MENU_SORT_DICTIONARY, MyFrame::OnTools)
 	EVT_MENU(MENU_COMPILE_MBROLA, MyFrame::OnTools)
+	EVT_MENU(MENU_COMPILE_INTONATION, MyFrame::OnTools)
 	EVT_MENU(MENU_CLOSE_ALL, MyFrame::OnQuit)
 	EVT_MENU(MENU_QUIT, MyFrame::OnQuit)
 	EVT_MENU(MENU_SPEAK_TRANSLATE, MyFrame::OnSpeak)
 	EVT_MENU(MENU_SPEAK_RULES, MyFrame::OnSpeak)
+	EVT_MENU(MENU_SPEAK_IPA, MyFrame::OnSpeak)
 	EVT_MENU(MENU_SPEAK_TEXT, MyFrame::OnSpeak)
 	EVT_MENU(MENU_SPEAK_FILE, MyFrame::OnSpeak)
 	EVT_MENU(MENU_SPEAK_STOP, MyFrame::OnSpeak)
@@ -218,9 +229,10 @@ BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
 	EVT_MENU(MENU_VOWELCHART2, MyFrame::OnTools)
 	EVT_MENU(MENU_VOWELCHART3, MyFrame::OnTools)
 	EVT_MENU(MENU_LEXICON_RU, MyFrame::OnTools)
+	EVT_MENU(MENU_LEXICON_BG, MyFrame::OnTools)
 	EVT_MENU(MENU_LEXICON_DE, MyFrame::OnTools)
 	EVT_MENU(MENU_LEXICON_IT, MyFrame::OnTools)
-	EVT_MENU(MENU_LEXICON_IT2, MyFrame::OnTools)
+	EVT_MENU(MENU_LEXICON_TEST, MyFrame::OnTools)
 	EVT_MENU(MENU_TO_UTF8, MyFrame::OnTools)
 	EVT_MENU(MENU_COUNT_WORDS, MyFrame::OnTools)
 	EVT_MENU(MENU_TEST, MyFrame::OnTools)
@@ -228,10 +240,18 @@ BEGIN_EVENT_TABLE(MyFrame, wxMDIParentFrame)
 
 	EVT_TIMER(1, MyFrame::OnTimer)
 	EVT_SIZE(MyFrame::OnSize)
-	EVT_SASH_DRAGGED_RANGE(ID_WINDOW_TOP, ID_WINDOW_BOTTOM, MyFrame::OnSashDrag)
+	EVT_SASH_DRAGGED(ID_WINDOW_LEFT, MyFrame::OnSashDrag)
 END_EVENT_TABLE()
 
 
+MyFrame::~MyFrame(void)
+{//====================
+	myframe->Show(false);
+	myframe->Iconize(false);   // os=Windows, get the non-iconsized size
+	myframe->GetPosition(&frame_x, &frame_y);
+	myframe->GetSize(&frame_w, &frame_h);
+}
+
 MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size,
     const long style):
   wxMDIParentFrame(parent, id, title, pos, size, style)
@@ -242,22 +262,16 @@ MyFrame::MyFrame(wxWindow *parent, const wxWindowID id, const wxString& title, c
 	int result;
 	int param;
 
-  // Create some dummy layout windows
-wxSashLayoutWindow *win;
-
   // Another window to the left of the client window
-  win = new wxSashLayoutWindow(this, ID_WINDOW_LEFT2,
+  m_leftWindow = new wxSashLayoutWindow(this, ID_WINDOW_LEFT,
                                wxDefaultPosition, wxSize(298, 30),
                                wxNO_BORDER | wxSW_3D | wxCLIP_CHILDREN);
-  win->SetDefaultSize(wxSize(310, 1000));
-  win->SetOrientation(wxLAYOUT_VERTICAL);
-  win->SetAlignment(wxLAYOUT_LEFT);
-//  win->SetBackgroundColour(wxColour(0, 255, 255));
-  win->SetSashVisible(wxSASH_RIGHT, TRUE);
+  m_leftWindow->SetDefaultSize(wxSize(310, 1000));
+  m_leftWindow->SetOrientation(wxLAYOUT_VERTICAL);
+  m_leftWindow->SetAlignment(wxLAYOUT_LEFT);
+  m_leftWindow->SetSashVisible(wxSASH_RIGHT, TRUE);
 
-  m_leftWindow2 = win;
-
-	notebook = new wxNotebook(m_leftWindow2,-1);
+	notebook = new wxNotebook(m_leftWindow,-1);
 //	notebook->AddPage(voicedlg,_T("Voice"),FALSE);
 	formantdlg = new FormantDlg(notebook);
 	notebook->AddPage(formantdlg,_T(" Spect"),FALSE);
@@ -267,7 +281,6 @@ wxSashLayoutWindow *win;
 	notebook->AddPage(transldlg,_T("Text"),TRUE);
 
 
-	ConfigInit();
 	LoadConfig();
 	WavegenInitSound();
 
@@ -319,12 +332,14 @@ void MyFrame::SetVoiceTitle(char *voice_name)
 {//==========================================
 	char buf[100];
 	SetTitle(AppName + _T(" - ") + wxString(voice_name,wxConvLocal) + _T("  voice"));
-	if(data_menu != NULL)
+	if((data_menu != NULL) && (translator != NULL))
 	{
-		sprintf(buf,"Compile &dictionary '%s'",dictionary_name);
+		sprintf(buf,"Compile &dictionary '%s'",translator->dictionary_name);
 		data_menu->SetLabel(MENU_COMPILE_DICT, wxString(buf,wxConvLocal));
-		sprintf(buf,"&Layout '%s_rules' file",dictionary_name);
+		sprintf(buf,"&Layout '%s_rules' file",translator->dictionary_name);
 		data_menu->SetLabel(MENU_FORMAT_DICTIONARY, wxString(buf,wxConvLocal));
+		sprintf(buf,"&Sort '%s_rules' file",translator->dictionary_name);
+		data_menu->SetLabel(MENU_SORT_DICTIONARY, wxString(buf,wxConvLocal));
 	}
 }
 
@@ -460,8 +475,8 @@ void OnOptions2(int event_id)
 
 	switch(event_id)
 	{
-	case MENU_PARAMS:
-		value = wxGetNumberFromUser(_T(""),_T(""),_T("Speed"),option_speed,80,390);
+	case MENU_OPT_SPEED:
+		value = wxGetNumberFromUser(_T(""),_T(""),_T("Speed"),option_speed,80,500);
 		if(value > 0)
 		{
 			option_speed = value;
@@ -470,6 +485,18 @@ void OnOptions2(int event_id)
 		}
 		break;
 
+	case MENU_OPT_PUNCT:
+		transldlg->t_source->SetValue(_T("<tts:style field=\"punctuation\" mode=\"all\">\n"));
+		transldlg->t_source->SetInsertionPointEnd();
+		notebook->SetSelection(1);
+		break;
+
+	case MENU_OPT_SPELL:
+		transldlg->t_source->SetValue(_T("<say-as interpret-as=\"tts:char\">\n"));
+		transldlg->t_source->SetInsertionPointEnd();
+		notebook->SetSelection(1);
+		break;
+
 	case MENU_PATH0:
 		string = wxFileSelector(_T("Master phonemes file"),wxFileName(path_phfile).GetPath(),
 			_T(""),_T(""),_T("*"),wxOPEN);
@@ -524,6 +551,32 @@ void MyFrame::OnOptions(wxCommandEvent& event)
 }
 
 
+void DisplayErrorFile(const char *fname)
+{//=====================================
+	int len;
+	FILE *f;
+	char *msg;
+	wxString msg_string;
+
+	len = GetFileLength(fname);
+	if(len > 0)
+	{
+		if(len > 1500)
+			len = 1500;   // restrict length to pppppevent crash in wxLogMessage()
+		msg = (char *)malloc(len+1);
+		if(msg != NULL)
+		{
+			f = fopen(fname,"r");
+			len = fread(msg,1, len, f);
+			fclose(f);
+			msg[len] = 0;
+			msg_string = wxString(msg,wxConvUTF8);
+			wxLogMessage(msg_string);
+			free(msg);
+		}
+	}
+}  // end of DisplayErrorFile
+
 
 
 void MyFrame::OnTools(wxCommandEvent& event)
@@ -553,9 +606,10 @@ void MyFrame::OnTools(wxCommandEvent& event)
 		break;
 
 	case MENU_LEXICON_RU:
+	case MENU_LEXICON_BG:
 	case MENU_LEXICON_DE:
 	case MENU_LEXICON_IT:
-	case MENU_LEXICON_IT2:
+	case MENU_LEXICON_TEST:
 		CompareLexicon(event.GetId());  // Compare a lexicon with _rules translation
 		break;
 
@@ -566,6 +620,10 @@ void MyFrame::OnTools(wxCommandEvent& event)
 	case MENU_COMPILE_MBROLA:
 		CompileMbrola();
 		break;
+
+	case MENU_COMPILE_INTONATION:
+		CompileIntonation();
+		break;
 		
 	case MENU_COMPILE_DICT_DEBUG:
 		debug_flag =1;  // and drop through to next case
@@ -573,8 +631,8 @@ void MyFrame::OnTools(wxCommandEvent& event)
 		sprintf(fname_log,"%s%s",path_dsource,"dict_log");
 		log = fopen(fname_log,"w");
 
-		LoadDictionary(translator, dictionary_name, 0);
-		if((err = CompileDictionary(path_dsource,dictionary_name,log,err_fname,debug_flag)) < 0)
+		LoadDictionary(translator, translator->dictionary_name, 0);
+		if((err = CompileDictionary(path_dsource,translator->dictionary_name,log,err_fname,debug_flag)) < 0)
 		{
 			wxLogError(_T("Can't access file:\n")+wxString(err_fname,wxConvLocal));
 
@@ -596,28 +654,7 @@ void MyFrame::OnTools(wxCommandEvent& event)
 			if(err > 0)
 			{
 				// display the error messages
-				int len;
-				FILE *f;
-				char *msg;
-				wxString msg_string;
-
-				len = GetFileLength(fname_log);
-				if(len > 0)
-				{
-					if(len > 2000)
-						len = 2000;   // restrict length to pppppevent crash in wxLogMessage()
-					msg = (char *)malloc(len+1);
-					if(msg != NULL)
-					{
-						f = fopen(fname_log,"r");
-						fread(msg,len,1,f);
-						fclose(f);
-						msg[len] = 0;
-						msg_string = wxString(msg,wxConvUTF8);
-						wxLogMessage(msg_string);
-						free(msg);
-					}
-				}
+				DisplayErrorFile(fname_log);
 			}
 		}
 		break;
@@ -654,11 +691,13 @@ void MyFrame::OnSpeak(wxCommandEvent& event)
 	{
 	case MENU_SPEAK_TRANSLATE:
 	case MENU_SPEAK_RULES:
+	case MENU_SPEAK_IPA:
 	case MENU_SPEAK_TEXT:
 		transldlg->OnCommand(event);
 		break;
 
 	case MENU_SPEAK_FILE:
+		out_ptr = NULL;
 		transldlg->SpeakFile();
 		break;
 
@@ -667,11 +706,14 @@ void MyFrame::OnSpeak(wxCommandEvent& event)
 		break;
 
 	case MENU_SPEAK_PAUSE:
+		out_ptr = NULL;
 		SpeakNextClause(NULL,NULL,3);
 		if(SynthStatus() & 2)
 			speak_menu->SetLabel(MENU_SPEAK_PAUSE,_T("&Resume"));
 		else
+		{
 			speak_menu->SetLabel(MENU_SPEAK_PAUSE,_T("&Pause"));
+		}
 		break;
 
 	case MENU_SPEAK_VOICE:
@@ -689,37 +731,26 @@ void MyFrame::OnSpeak(wxCommandEvent& event)
 
 void MyFrame::OnSashDrag(wxSashEvent& event)
 {
-    if (event.GetDragStatus() == wxSASH_STATUS_OUT_OF_RANGE)
-        return;
+	int w, h;
 
-    switch (event.GetId())
-    {
-        case ID_WINDOW_TOP:
-        {
-            m_topWindow->SetDefaultSize(wxSize(1000, event.GetDragRect().height));
-            break;
-        }
-        case ID_WINDOW_LEFT1:
-        {
-            m_leftWindow1->SetDefaultSize(wxSize(event.GetDragRect().width, 1000));
-            break;
-        }
-        case ID_WINDOW_LEFT2:
-        {
-            m_leftWindow2->SetDefaultSize(wxSize(event.GetDragRect().width, 1000));
-            break;
-        }
-        case ID_WINDOW_BOTTOM:
-        {
-            m_bottomWindow->SetDefaultSize(wxSize(1000, event.GetDragRect().height));
-            break;
-        }
-    }
-    wxLayoutAlgorithm layout;
-    layout.LayoutMDIFrame(this);
+	if (event.GetDragStatus() == wxSASH_STATUS_OUT_OF_RANGE)
+		return;
 
-    // Leaves bits of itself behind sometimes
-    GetClientWindow()->Refresh();
+	GetClientSize(&w, &h);
+
+	switch (event.GetId())
+	{
+		case ID_WINDOW_LEFT:
+		{
+			m_leftWindow->SetDefaultSize(wxSize(event.GetDragRect().width, h));
+			break;
+		}
+	}
+	wxLayoutAlgorithm layout;
+	layout.LayoutMDIFrame(this);
+
+	// Leaves bits of itself behind sometimes
+	GetClientWindow()->Refresh();
 }
 
 
