@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006 to 2010 by Jonathan Duddington                     *
+ *   Copyright (C) 2006 to 2011 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -134,7 +134,13 @@ int TestSynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 			fprintf(f_events,"'%s'\n",events->id.name);
 		else
 		if(type==espeakEVENT_PHONEME)
-			fprintf(f_events,"[%s]\n",WordToString(events->id.number));
+		{
+			char buf[10];
+			fprintf(f_events,"[%s]\n",WordToString(events->id.number));  //old version, only 4 characters bytes
+//			memcpy(buf, events->id.string, 8);
+//			buf[8] = 0;
+//			fprintf(f_events,"[%s]\n", buf);
+		}
 		else
 			fprintf(f_events,"%d\n",events->id.number);
 
@@ -200,8 +206,6 @@ static const unsigned short KOI8_R[0x60] = {
 
 
 #define N_CHARS  34
-int *p_unicode;
-int unicode[80];
 #define PH(c1,c2)  (c2<<8)+c1          // combine two characters into an integer for phoneme name 
 
 
@@ -291,11 +295,14 @@ static void DecodePhonemes2(const char *inptr, char *outptr)
 
 void Lexicon_It(int pass)
 {//======================
+// Reads a lexicon of pairs of words: normal spelling and spelling with accents
+// Creates file: dictsource/it_listx   which includes corrections for stress position and [E]/[e] and [O]/[o] phonemes
+// Words which are still in error are listed in file:  it_compare  (in the directory of the lexicon file).
 	int count=0;
 	int matched=0;
 	int ix;
 	int c;
-	char *p;
+	char *p, *p2;
 	int len;
 	int vowel_ix;
 	int stress_posn1;
@@ -307,8 +314,11 @@ void Lexicon_It(int pass)
 	FILE *f_out;
 	FILE *f_listx;
 	FILE *f_list_in = NULL;
+	int listx_count;
 	long int displ;
 	const char *alt_string;
+	wxString str;
+	static wxString fname_lex;
 	char buf[200];
 	char word[80];
 	char word1[80];
@@ -317,7 +327,11 @@ void Lexicon_It(int pass)
 	char temp[80];
 	char phonemes[80];
 	char phonemes2[80];
-	char buf_out[120];
+	char phonemes3[80];
+	char buf_out[200];
+	char buf_error[200];
+	char last_listx[200];
+
 	static const char *vowels1 = "aeiou";
 	static const char *vowels2 = "aeou";
 
@@ -335,31 +349,42 @@ void Lexicon_It(int pass)
 
 	static const char *exceptions[] = {ex1, ex2, ex3, ex4, ex5, ex6, ex7, ex8, ex9, ex10, NULL};
 
-	wxString fname = wxFileSelector(_T("Italian Lexicon"),path_dir1,_T(""),_T(""),_T("*"),wxOPEN);
+	if(pass == 1)
+	{
+		fname_lex = wxFileSelector(_T("Italian Lexicon"),path_dir1,_T(""),_T(""),_T("*"),wxOPEN);
+	}
 
-	strcpy(buf,fname.mb_str(wxConvLocal));
+	strcpy(buf,fname_lex.mb_str(wxConvLocal));
 	if((f_in = fopen(buf,"r")) == NULL)
 	{
-		wxLogError(_T("Can't read file ")+fname);
+		wxLogError(_T("Can't read file ")+fname_lex);
 		return;
 	}
-	path_dir1 = wxFileName(fname).GetPath();
+	path_dir1 = wxFileName(fname_lex).GetPath();
+	strcpy(buf_out, path_dir1.mb_str(wxConvLocal));
+	sprintf(buf, "%s/IT_errors", buf_out);
 	
-	if((f_out = fopen("compare_it","w")) == NULL)
+	if((f_out = fopen(buf,"w")) == NULL)
 	{
-		wxLogError(_T("Can't write file: compare_it "));
+		str = wxString(buf, wxConvLocal);
+		wxLogError(_T("Can't write file: ") + str);
 		return;
 	}
 
+	listx_count = 0;
+	last_listx[0] = 0;
+
 	if(pass == 1)
 	{
 		sprintf(buf,"%s/it_listx",path_dsource);
 		remove(buf);
 		CompileDictionary(path_dsource,"it",NULL,NULL,0);
 		f_listx = fopen(buf,"w");
+		wxLogStatus(_T("Pass 1"));
 	}
 	else
 	{
+		CompileDictionary(path_dsource,"it",NULL,NULL,0);
 		sprintf(buf,"%s/it_listx2",path_dsource);
 		f_listx = fopen(buf,"w");
 		sprintf(buf,"%s/it_listx",path_dsource);
@@ -392,9 +417,9 @@ void Lexicon_It(int pass)
 			continue;
 
 		if(strlen(word) < 8)
-			sprintf(buf_out,"%s\t\t%s\t",word,temp);
+			sprintf(buf_error,"%s\t\t%s\t",word,temp);
 		else
-			sprintf(buf_out,"%s\t%s",word,temp);
+			sprintf(buf_error,"%s\t%s",word,temp);
 
 		sprintf(word1," %s  ",word);
 
@@ -437,6 +462,7 @@ void Lexicon_It(int pass)
 				break;
 			}
 		}
+
 		// translate
 		TranslateWord(translator,&word1[1],0, NULL);
 		DecodePhonemes(word_phonemes,phonemes);
@@ -552,23 +578,38 @@ void Lexicon_It(int pass)
 						sscanf(buf, "%s", word1);
 						if(strcmp(word1, word_stem) < 0)
 						{
-							fprintf(f_listx,"%s",buf);  // copy it_listx from pass 1 until we reach the matching word
-						}
-						else
-						if(strcmp(word1, word_stem) == 0)
-						{
-							p = buf;
-							while((*p != '\n') && (*p != 0)) *p++;
-							*p = 0;
-							fprintf(f_listx,"%s %s\n",buf,alt_string);   // add $alt or $alt2 to the entry
-							break;
+							sprintf(buf_out,"%s",buf);  // copy it_listx from pass 1 until we reach the matching word
 						}
 						else
 						{
-							fprintf(f_listx,"%s %s\n", word_stem, alt_string);   // add a new word with $alt or $alt2
-							fseek(f_list_in, displ, SEEK_SET);
+							if(strcmp(word1, word_stem) == 0)
+							{
+								p = buf;
+								while((*p != '\n') && (*p != 0)) *p++;
+								*p = 0;
+								sprintf(buf_out,"%s %s\n",buf,alt_string);   // add $alt or $alt2 to the entry
+							}
+							else
+							{
+								sprintf(buf_out,"%s %s\n", word_stem, alt_string);   // add a new word with $alt or $alt2
+								fseek(f_list_in, displ, SEEK_SET);
+							}
+
+							if(strcmp(buf_out, last_listx) != 0)
+							{
+								fprintf(f_listx, "%s", buf_out);
+								listx_count++;
+								strcpy(last_listx, buf_out);
+							}
 							break;
 						}
+
+						if(strcmp(buf_out, last_listx) != 0)
+						{
+							fprintf(f_listx, "%s", buf_out);
+							listx_count++;
+							strcpy(last_listx, buf_out);
+						}
 					}
 				}
 			}
@@ -576,7 +617,26 @@ void Lexicon_It(int pass)
 		}
 		else
 		{
-			fprintf(f_out,"%s\t%s\t%s\n",buf_out,phonemes,phonemes2);
+			// allow if the only difference is no primary stress
+			p2 = phonemes2;
+			p = phonemes3;
+			while(*p2 != 0)
+			{
+				*p = *p2++;
+				if((*p2 == ':') && (strchr("aeiouEO", *p) != NULL)) p2++;  // change lone vowels to short by removing ':'
+				if(*p == '\'') *p = ',';  // change primary to secondary stress
+				p++;
+			}
+			*p = 0;
+			if(strcmp(phonemes, phonemes3) == 0)
+			{
+				matched++;
+			}
+			else
+			{
+				// still doesn't match, report this word
+				fprintf(f_out,"%s\t%s\t%s\n",buf_error,phonemes,phonemes2);
+			}
 		}
 	}
 
@@ -584,15 +644,32 @@ void Lexicon_It(int pass)
 	{
 		while(fgets(buf, sizeof(buf), f_list_in) != NULL)
 		{
-			fprintf(f_listx, "%s", buf);  // copy the remaining entries from pass 1
+			if(strcmp(buf, last_listx) != 0)  // check for duplicate entries
+			{
+				fprintf(f_listx, "%s", buf);  // copy the remaining entries from pass 1
+				listx_count++;
+				strcpy(last_listx, buf);
+			}
 		}
 		fclose(f_list_in);
 	}
 	fclose(f_in);
 	fclose(f_out);
 	fclose(f_listx);
-	wxLogStatus(_T("Completed, equal=%d  different=%d"),matched,count-matched);
-}
+
+	if(pass == 2)
+	{
+		sprintf(buf,"%s/it_listx",path_dsource);
+		remove(buf);
+		sprintf(buf_out,"%s/it_listx2",path_dsource);
+		rename(buf_out, buf);
+		wxLogStatus(_T("Created file 'it_listx', entries=%d  errors=%d  total words=%d"),listx_count, count-matched, count);
+	}
+	else
+	{
+		wxLogStatus(_T("Pass 1, equal=%d  different=%d"),matched,count-matched);
+	}
+}  // end of Lexicon_It
 
 
 void Lexicon_De()
@@ -762,33 +839,317 @@ void Lexicon_De()
 	fclose(f_in);
 	fclose(f_out);
 	wxLogStatus(_T("Completed, equal=%d  different=%d"),matched,count-matched);
-}
+}  // end of Lexicon_De
+
+
+extern int IsVowel(Translator *tr, int letter);
+
+void Lexicon_Test()
+{//================
+	int c1, c2, c3;
+	char *p;
+	int prev_c1=0;
+	int prev_c2=0;
+	int prev_c3 = 0;
+	FILE *f_in;
+	FILE *f_out;
+	char buf[200];
+
+	wxString s_fname = wxFileSelector(_T("List of UTF-8 words with Combining Grave Accent U+300 to indicate stress"),path_dir1,
+		_T(""),_T(""),_T("*"),wxOPEN);
+	if(s_fname.IsEmpty())
+		return;
+	strcpy(buf,s_fname.mb_str(wxConvLocal));
+	path_dir1 = wxFileName(s_fname).GetPath();
+
+	if((f_in = fopen(buf,"r")) == NULL)
+	{
+		wxLogError(_T("Can't read file: ") + wxString(buf,wxConvLocal));
+		return;
+	}
+
+	strcat(buf,"_1");
+	if((f_out = fopen(buf,"w")) == NULL)
+	{
+		wxLogError(_T("Can't write file: ") + wxString(buf,wxConvLocal));
+		fclose(f_in);
+		return;
+	}
+
+	while(!feof(f_in))
+	{
+		if((p = fgets(buf,sizeof(buf),f_in)) == NULL)
+			break;
+
+		if(buf[0] == 0)
+			continue;
+
+		p += utf8_in(&c1, p);
+		p += utf8_in(&c2, p);
+		p += utf8_in(&c3, p);
+
+		c1 = towlower(c1);
+		c2 = towlower(c2);
+		c3 = towlower(c3);
+
+		if(IsVowel(translator, c1))
+			continue;
+		if(IsVowel(translator, c2))
+			continue;
+
+		if((prev_c1 != c1) || (prev_c2 != c2) || ((prev_c3 != c3) && !IsVowel(translator,c3)))
+			fputc('\n',f_out);
+		prev_c1 = c1;
+		prev_c2 = c2;
+		prev_c3 = c3;
+		fprintf(f_out,"%s",buf);
+
+	}
+	fclose(f_in);
+	fclose(f_out);
+}  // end of Lexicon_Test
+
+
+
+void Lexicon_Bg()
+{//==============
+// Bulgarian: compare stress markup in a list of words with lookup using bg_rules
+
+	char *p;
+	char *pw;
+	char *pw1;
+	int cc;
+	int vcount;
+	int lex_stress;
+	int input_length;
+	int n_words=0;
+	int n_wrong=0;
+	int n_out=0;
+	int n_stress;
+	int max_stress;
+	int max_stress_posn;
+	int stress_first;
+	int done;
+	PHONEME_TAB *ph;
+
+	FILE *f_in;
+	FILE *f_out;
+	
+	char word[80];
+	char word_in[80];
+	char phonemes[N_WORD_PHONEMES];
+	char buf[200];
+	char fname[sizeof(path_dsource)+20];
+
+	static unsigned short bg_vowels[] = {0x430, 0x435, 0x438, 0x43e, 0x443, 0x44a, 0x44d, 0x44e, 0x44f, 0x450, 0x451, 0x45d, 0};
+
+	if(gui_flag == 0)
+		return;
+
+	wxString s_fname = wxFileSelector(_T("List of UTF-8 words with Combining Grave Accent U+300 to indicate stress"),path_dir1,
+		_T(""),_T(""),_T("*"),wxOPEN);
+	if(s_fname.IsEmpty())
+		return;
+	strcpy(buf,s_fname.mb_str(wxConvLocal));
+	path_dir1 = wxFileName(s_fname).GetPath();
+
+	if((f_in = fopen(buf,"r")) == NULL)
+	{
+		wxLogError(_T("Can't read file: ") + wxString(buf,wxConvLocal));
+		return;
+	}
+	input_length = GetFileLength(buf);
+
+	sprintf(fname,"%s%c%s",path_dsource,PATHSEP,"bg_listx");
+	remove(fname);
+	CompileDictionary(path_dsource,"bg",NULL,NULL,0);
+
+	if((f_out = fopen(fname,"w")) == NULL)
+	{
+		wxLogError(_T("Can't write to: ")+wxString(fname,wxConvLocal));
+		fclose(f_in);
+		return;
+	}
+
+	LoadVoice("bg",0);
+	progress = new wxProgressDialog(_T("Lexicon"),_T(""),input_length);
+
+	for(;;)
+	{
+		if((n_words & 0x3ff) == 0)
+		{
+			progress->Update(ftell(f_in));
+		}
+
+		if(fgets(buf,sizeof(buf),f_in) == NULL)
+			break;
+
+		if(isspace2(buf[0]))
+			continue;
+
+		// convert from UTF-8 to Unicode
+		word[0] = 0;
+		word[1] = ' ';
+		pw = &word[2];
+		pw1 = word_in;
+		p = buf;
+		while(*p == ' ') p++;
+		vcount = 0;
+		lex_stress = 0;
+		n_stress = 0;
+		stress_first = 0;
+
+		// find the marked stress position
+		for(;;)
+		{
+			p += utf8_in(&cc, p);
+			if(iswspace(cc))
+				break;
+			if(cc == 0xfeff)
+				continue;   // ignore UTF-8 indication
+
+			if(cc == '`')
+				cc = '\'';
+			pw1 += utf8_out(towlower(cc), pw1);   // copy UTF-8 to 'word_in'
+
+			if(lookupwchar(bg_vowels, cc) != 0)
+				vcount++;
+
+			if((cc == '\'') || (cc == 0x300) || (cc == 0x450) || (cc == 0x45d))
+			{
+				// backprime (before the vowel), combining grave accent, or accented vowel character
+				if(cc == '\'')
+					lex_stress = vcount+1;
+				else
+					lex_stress = vcount;
+				n_stress++;
+
+				if(vcount == 1)
+					stress_first = 1;
+
+				if((cc == '\'') || (cc == 0x300))
+					continue;		// discard backprime or combining accent
+				if(cc == 0x450)
+					cc = 0x435;		// remove accent from vowel
+				if(cc == 0x45d)
+					cc = 0x438;
+			}
+
+			pw += utf8_out(cc, pw);   // copy UTF-8 to 'word'
+		}
+		*pw++ = ' ';
+		*pw = 0;
+		*pw1 = 0;
+
+		// translate
+		TranslateWord(translator, &word[2],0, NULL);
+		DecodePhonemes(word_phonemes,phonemes);
+
+		// find the stress position in the translation
+		max_stress = 0;
+		max_stress_posn = -1;
+		vcount = 0;
+
+		ph = phoneme_tab[phonPAUSE];
+		for(p=word_phonemes; *p != 0; p++)
+		{
+			ph = phoneme_tab[(unsigned int)*p];
+			if(ph == NULL)
+				continue;
+
+			if(ph->type == phVOWEL)
+				vcount++;
+			if(ph->type == phSTRESS)
+			{
+				if(ph->std_length > max_stress)
+				{
+					max_stress = ph->std_length;
+					max_stress_posn = vcount+1;
+				}
+			}
+		}
+
+if(n_stress > 1) n_stress = 1;
+		done = 0;
+
+		if(vcount < 2)
+		{
+			// don't list words with only one vowel
+		}
+		else
+		if((lex_stress != max_stress_posn) || (n_stress != 1))
+		{
+			if((vcount > 0) && (lex_stress > 0) && (lex_stress <= 7))
+			{
+				if((n_stress == 2) && (stress_first))
+				{
+					done = 1;
+					fprintf(f_out,"%s\t$%d\n",&word[2],lex_stress);
+				}
+				if(n_stress == 1)
+				{
+					done = 1;
+					fprintf(f_out,"%s\t$%d\n",&word[2],lex_stress);
+				}
+			}
+
+			if(done == 0)
+			{
+				n_wrong++;
+				fprintf(f_out,"// %s\t$text %s\n", &word[2], word_in);
+			}
+
+			if(done)
+				n_out++;
+		} 
+
+		n_words++;
+	}
+
+	fclose(f_in);
+	fclose(f_out);
+
+	CompileDictionary(path_dsource,"bg",NULL,NULL,0);
+
+
+	delete progress;
+	sprintf(buf,"Lexicon: Input %d,  Output %d,  $text %d",n_words,n_out,n_wrong);
+	wxLogStatus(wxString(buf,wxConvLocal));
+}  // end of Lexicon_Bg
+
 
 
 void Lexicon_Ru()
 {//==============
-	// compare stress markings in Russian RuLex file with lookup in ru_rules
+// compare stress markings in Russian RuLex file with lookup in ru_rules
+
+// Input file contains a list of Russian words (UTF-8), one per line
+// Stress position can be indicated either by:
+// A $ sign and syllable number after the word, eg:
+//   абажура  $3
+// or by a + sign after the stressed vowel, eg:
+//   абажу+ра
+
+// espeakedit produces a file:  dictsource/ru_listx  and a log file dictsource/ru_log
+
 	int ix;
 	char *p;
 	int  c;
 	FILE *f_in;
 	FILE *f_out;
 	FILE *f_log;
-	FILE *f_roots;
 	PHONEME_TAB *ph;
+	int ph_code;
 	int vcount;
 	int ru_stress;
 	int max_stress;
 	int max_stress_posn;
 	int n_words=0;
 	int n_wrong=0;
+	int n_errors=0;
 	int wlength;
 	int input_length;
 
-	int sfx;
-	const char *suffix;
-	int wlen;
-	int len;
 	int check_root;
 
 	char word[80];
@@ -799,14 +1160,20 @@ void Lexicon_Ru()
 	char buf[200];
 	char fname[sizeof(path_dsource)+20];
 
-	// KOI8-R codes for Russian vowels
-	static unsigned char vowels[] = {0xa3,0xc0,0xc1,0xc5,0xc9,0xcf,0xd1,0xd5,0xd9,0xdc,0};
+	// character codes for Russian vowels
+	static unsigned short ru_vowels[] = {0x430,0x435,0x438,0x439,0x43e,0x443,0x44d,0x44e,0x44f,0x450,0x451,0};
 
 	typedef struct {
 		const char *suffix;
 		int  syllables;
 	} SUFFIX;
 
+#ifdef deleted
+	FILE *f_roots;
+	int sfx;
+	const char *suffix;
+	int wlen;
+	int len;
 	static SUFFIX suffixes[] = {
 {NULL,0},
 	{"ичу",2},
@@ -826,7 +1193,7 @@ void Lexicon_Ru()
 	{"и",1},
 
 	{NULL,0}};
-
+#endif
 
 	memset(counts,0,sizeof(counts));
 
@@ -853,7 +1220,12 @@ void Lexicon_Ru()
 	}
 	input_length = GetFileLength(buf);
 
-	sprintf(fname,"%s%c%s",path_dsource,PATHSEP,"ru_listx_1");
+	sprintf(fname,"%s%c%s",path_dsource,PATHSEP,"ru_listx");
+	remove(fname);
+
+	// compile ru_dict without ru_listx
+	CompileDictionary(path_dsource,"ru",NULL,NULL,0);
+
 	if((f_out = fopen(fname,"w")) == NULL)
 	{
 		wxLogError(_T("Can't write to: ")+wxString(fname,wxConvLocal));
@@ -864,7 +1236,7 @@ void Lexicon_Ru()
 	sprintf(fname,"%s%c%s",path_dsource,PATHSEP,"ru_log");
 	f_log = fopen(fname,"w");
 	sprintf(fname,"%s%c%s",path_dsource,PATHSEP,"ru_roots_1");
-	f_roots = fopen(fname,"w");
+//	f_roots = fopen(fname,"w");
 
 	LoadVoice("ru",0);
 
@@ -884,46 +1256,46 @@ void Lexicon_Ru()
 		if(fgets(buf,sizeof(buf),f_in) == NULL)
 			break;
 
-		if(isspace2(buf[0]))
-			continue;
+		if((p = strstr(buf,"//")) != NULL)
+			*p = '\n';   // truncate at comment
 
-		// convert word from KOI8-R to UTF8
 		p = buf;
+		while((*p == ' ') || (*p == '\t')) p++;
+		if(*p == '\n')
+			continue;  // blank line
+
 		ix = 0;
 		wlength = 0;
-p_unicode = unicode;
-		while(!isspace2(c = (*p++ & 0xff)))
+		vcount = 0;
+		ru_stress = -1;
+
+		for(;;)
 		{
-			if(c >= 0xa0)
-			{
-				c = KOI8_R[c-0xa0];
-*p_unicode++ = c;
-			}
+			p += utf8_in(&c, p);
+			if(isspace(c))
+				break;
 
-
-			wlength++;
-			ix += utf8_out(c,&word[ix]);
+			if(lookupwchar(ru_vowels, c))
+				vcount++;
+			if(c == '+')
+				ru_stress = vcount;
+			else
+				ix += utf8_out(c, &word[ix]);
 		}
 		word[ix] = 0;
-*p_unicode=0;
 
-		sprintf(word2," %s ",word);
+		sprintf(word2," %s ",word);   // surround word by spaces before calling TranslateWord()
 
-		// find the marked stress position
-		vcount = 0;
-		ru_stress = 0;
-		while(*p == ' ') p++;
-
-		while((c = (*p++ & 0xff)) != '\n')
+		// find the marked stress position, if it has not been marked by a + after the stressed vowel
+		if(ru_stress == -1)
 		{
-			if(c == '+')
+			while((*p == ' ') || (*p == '\t')) p++;
+			sscanf(p,"$%d",&ru_stress);
+			if(ru_stress == -1)
 			{
-				ru_stress = vcount;
-				break;
-			}
-			if(strchr((char *)vowels,c) != NULL)
-			{
-				vcount++;
+				n_errors++;
+				fprintf(f_log,"%s",buf);
+				continue;   // stress position not found
 			}
 		}
 
@@ -938,13 +1310,13 @@ p_unicode = unicode;
 		check_root = 0;
 
 		ph = phoneme_tab[phonPAUSE];
-		for(p=word_phonemes; *p != 0; p++)
+		for(p=word_phonemes; (ph_code = *p & 0xff) != 0; p++)
 		{
-			ph = phoneme_tab[(unsigned int)*p];
+			ph = phoneme_tab[ph_code];
 			if(ph == NULL)
 				continue;
 
-			if(ph->type == phVOWEL)
+			if((ph->type == phVOWEL) && (ph_code != phonSCHWA_SHORT))
 				vcount++;
 			if(ph->type == phSTRESS)
 			{
@@ -963,6 +1335,7 @@ p_unicode = unicode;
 			{
 				fprintf(f_log,"%s\t $%d\t // %s\n",word,ru_stress,phonemes);
 			}
+			n_errors++;
 		}
 		else
 		{
@@ -972,7 +1345,14 @@ p_unicode = unicode;
 			{
 				n_wrong++;
 				if((ru_stress==0) || (ru_stress > 7))
+				{
 					fprintf(f_out,"// ");   // we only have $1 to $7 to indicate stress position
+					if(f_log != NULL)
+					{
+						fprintf(f_log,"%s\t $%d\t // %s\n",word,ru_stress,phonemes);
+					}
+					n_errors++;
+				}
 				else
 					check_root = 1;
 
@@ -992,6 +1372,8 @@ p_unicode = unicode;
 			}
 		}
 
+
+#ifdef deleted
 		if(check_root)
 		{
 			// does this word match any suffixes ?
@@ -1014,13 +1396,14 @@ p_unicode = unicode;
 				}
 			}
 		}
+#endif
 	}
 
 	fclose(f_in);
 	fclose(f_out);
-	fclose(f_roots);
+//	fclose(f_roots);
 
-	sprintf(buf,"Lexicon: Total %d  OK %d  wrong %d",n_words,n_words-n_wrong,n_wrong);
+	sprintf(buf,"Lexicon: Total %d  OK %d  fixed %d  errors %d (see ru_log)",n_words,n_words-n_wrong,n_wrong,n_errors);
 	if(gui_flag)
 	{
 		delete progress;
@@ -1037,13 +1420,13 @@ p_unicode = unicode;
 #ifdef deleted
 		// list tables of frequency of stress position for words of different syllable lengths
 		int j,k;
-		for(ix=0; ix<12; ix++)
+		for(ix=2; ix<12; ix++)
 		{
-			fprintf(f_log,"%2d syl: ",ix);
+			fprintf(f_log,"%2d syllables\n",ix);
 			for(k=0; k<10; k++)
 			{
 				fprintf(f_log,"  %2d :",k);
-				for(j=0; j<10; j++)
+				for(j=1; j<=ix; j++)
 				{
 					fprintf(f_log,"%6d ",counts[ix][j][k]);
 				}
@@ -1066,15 +1449,19 @@ void CompareLexicon(int id)
 	case MENU_LEXICON_RU:
 		Lexicon_Ru();
 		break;
+	case MENU_LEXICON_BG:
+		Lexicon_Bg();
+		break;
 	case MENU_LEXICON_DE:
 		Lexicon_De();
 		break;
 	case MENU_LEXICON_IT:
 		Lexicon_It(1);
-		break;
-	case MENU_LEXICON_IT2:
 		Lexicon_It(2);
 		break;
+	case MENU_LEXICON_TEST:
+		Lexicon_Test();
+		break;
 	}
 }  // end of CompareLexicon
 
@@ -1928,11 +2315,12 @@ if(control==2)
 	fprintf(f_events,"Type             Audio  Text  Length Id\n");
 	fclose(f_events);
 
-	espeak_Initialize(AUDIO_OUTPUT_RETRIEVAL,1000,NULL,1);
+	espeak_Initialize(AUDIO_OUTPUT_RETRIEVAL, 1000, NULL, espeakINITIALIZE_PHONEME_EVENTS);
 	espeak_SetSynthCallback(TestSynthCallback);
 	espeak_SetUriCallback(TestUriCallback);
 
-  espeak_Synth(textbuf, strlen(textbuf)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)user_data);
+	espeak_SetVoiceByName("en");
+	espeak_Synth(textbuf, strlen(textbuf)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)user_data);
 //  espeak_Synth(text1, strlen(text1)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)(user_data+1));
 
   espeak_SetParameter(espeakPUNCTUATION, 1, 0);
