@@ -226,6 +226,7 @@ Converts synthesis parameters to a waveform.
 static int parwave(klatt_frame_ptr frame) 
 {
 	double temp;
+	int value;
 	double outbypas;
 	double out;
 	long n4;
@@ -420,7 +421,7 @@ if(option_log_frames)
 #endif
 
 		out = resonator(&(kt_globals.rsn[Rout]),out);
-		temp = (out * wdata.amplitude * kt_globals.amp_gain0) ;   /* Convert back to integer */
+		temp = (int)(out * wdata.amplitude * kt_globals.amp_gain0) ;   /* Convert back to integer */
 
 
 		// mix with a recorded WAV if required for this phoneme
@@ -457,18 +458,27 @@ if(option_log_frames)
 			temp = (temp * kt_globals.fadeout) / 64;
 		}
 
-		if (temp < -32768.0)
+		value = (int)temp + ((echo_buf[echo_tail++]*echo_amp) >> 8);
+		if(echo_tail >= N_ECHO_BUF)
+			echo_tail=0;
+
+		if (value < -32768)
 		{
-			temp = -32768.0;
+			value = -32768;
 		}
 	
-		if (temp > 32767.0)
+		if (value > 32767)
 		{
-			temp =  32767.0;
+			value =  32767;
 		}
 
-		*out_ptr++ = int(temp);
-		*out_ptr++ = int(temp) >> 8;
+		*out_ptr++ = value;
+		*out_ptr++ = value >> 8;
+
+		echo_buf[echo_head++] = value;
+		if(echo_head >= N_ECHO_BUF)
+			echo_head = 0;
+
 		sample_count++;
 		if(out_ptr >= out_end)
 		{
@@ -873,12 +883,10 @@ static void setzeroabc(long int f, long int bw, resonator_ptr rp)
 	double arg;
 	
 	f = -f;
-	
-	if(f>=0)
-	{
-		f = -1;
-	}
-	
+
+//NOTE, changes made 30.09.2011 for Reece Dunn <msclrhd@googlemail.com>
+// fix a sound spike when f=0
+
 	/* First compute ordinary resonator coefficients */
 	/* Let r  =  exp(-pi bw t) */
 	arg = kt_globals.minus_pi_t * bw;
@@ -895,9 +903,17 @@ static void setzeroabc(long int f, long int bw, resonator_ptr rp)
 	rp->a = 1.0 - rp->b - rp->c;
 	
 	/* Now convert to antiresonator coefficients (a'=1/a, b'=b/a, c'=c/a) */
-	rp->a = 1.0 / rp->a;
-	rp->c *= -rp->a;
-	rp->b *= -rp->a;
+	/* If f == 0 then rp->a gets set to 0 which makes a'=1/a set a', b' and c' to
+	 * INF, causing an audible sound spike when triggered (e.g. apiration with the
+	 * nasal register set to f=0, bw=0).
+	 */
+	if (rp->a != 0)
+	{
+		/* Now convert to antiresonator coefficients (a'=1/a, b'=b/a, c'=c/a) */
+		rp->a = 1.0 / rp->a;
+		rp->c *= -rp->a;
+		rp->b *= -rp->a;
+	}
 }
 
 
@@ -963,7 +979,7 @@ static double DBtoLIN(long dB)
 		return(0);
 	}
 	
-	return(double(amptable[dB]) * 0.001);
+	return((double)(amptable[dB]) * 0.001);
 }
 
 
@@ -1040,7 +1056,7 @@ int Wavegen_Klatt(int resume)
 		for(ix=0; ix < N_KLATTP; ix++)
 		{
 			klattp1[ix] += klattp_inc[ix];
-			klattp[ix] = int(klattp1[ix]);
+			klattp[ix] = (int)klattp1[ix];
 		}
 
 		for(ix=0; ix<=6; ix++)
@@ -1203,7 +1219,7 @@ if(option_log_frames)
 		else
 		{
 			klattp1[ix] = klattp[ix] = fr1->klattp[ix];
-			klattp_inc[ix] = double((fr2->klattp[ix] - klattp[ix]) * STEPSIZE)/length;
+			klattp_inc[ix] = (double)((fr2->klattp[ix] - klattp[ix]) * STEPSIZE)/length;
 		}
 
 		// get klatt parameter adjustments for the voice
@@ -1216,7 +1232,7 @@ if(option_log_frames)
 	for(ix=1; ix < 6; ix++)
 	{
 		peaks[ix].freq1 = (fr1->ffreq[ix] * v->freq[ix] / 256.0) + v->freqadd[ix];
-		peaks[ix].freq = int(peaks[ix].freq1);
+		peaks[ix].freq = (int)peaks[ix].freq1;
 		next = (fr2->ffreq[ix] * v->freq[ix] / 256.0) + v->freqadd[ix];
 		peaks[ix].freq_inc =  ((next - peaks[ix].freq1) * STEPSIZE) / length;
 
@@ -1224,7 +1240,7 @@ if(option_log_frames)
 		{
 			// klatt bandwidth for f1, f2, f3 (others are fixed)
 			peaks[ix].bw1 = fr1->bw[ix] * 2;
-			peaks[ix].bw = int(peaks[ix].bw1);
+			peaks[ix].bw = (int)peaks[ix].bw1;
 			next = fr2->bw[ix] * 2;
 			peaks[ix].bw_inc =  ((next - peaks[ix].bw1) * STEPSIZE) / length;
 		}
@@ -1235,7 +1251,7 @@ if(option_log_frames)
 	if(peaks[0].freq1 == 0)
 		peaks[0].freq1 = kt_frame.Fhz[F_NP];   // if no nasal zero, set it to same freq as nasal pole
 
-	peaks[0].freq = int(peaks[0].freq1);
+	peaks[0].freq = (int)peaks[0].freq1;
 	next = fr2->klattp[KLATT_FNZ] * 2;
 	if(next == 0)
 		next = kt_frame.Fhz[F_NP];
@@ -1252,12 +1268,12 @@ if(option_log_frames)
 		for(ix=1; ix < 7; ix++)
 		{
 			peaks[ix].bp1 = fr1->klatt_bp[ix] * 4;  // parallel bandwidth
-			peaks[ix].bp = int(peaks[ix].bp1);
+			peaks[ix].bp = (int)peaks[ix].bp1;
 			next = fr2->klatt_bp[ix] * 2;
 			peaks[ix].bp_inc =  ((next - peaks[ix].bp1) * STEPSIZE) / length;
 
 			peaks[ix].ap1 = fr1->klatt_ap[ix];   // parallal amplitude
-			peaks[ix].ap = int(peaks[ix].ap1);
+			peaks[ix].ap = (int)peaks[ix].ap1;
 			next = fr2->klatt_ap[ix] * 2;
 			peaks[ix].ap_inc =  ((next - peaks[ix].ap1) * STEPSIZE) / length;
 		}
