@@ -35,6 +35,7 @@
 #include "phoneme.h"
 #include "synthesize.h"
 #include "voice.h"
+#include "translate.h"
 #include "spect.h"
 #include "options.h"
 
@@ -246,6 +247,7 @@ void SpectDisplay::RefreshDialogValues(int type)
 //{=============================================
 {
 	int ix;
+	long int value;
 	SpectFrame *sf;
 
 	if(spectseq == NULL)
@@ -255,7 +257,13 @@ void SpectDisplay::RefreshDialogValues(int type)
 
 	spectseq->amplitude = formantdlg->t_amplitude->GetValue();
 	sf->amp_adjust = formantdlg->t_ampframe->GetValue();
-	sf->length_adjust = formantdlg->t_timeframe->GetValue() - spectseq->GetFrameLength(sframe,0,NULL);
+
+	if(use_spin_controls)
+		value = formantdlg->t_timeframe->GetValue();
+	else
+		formantdlg->tt_timeframe->GetValue().ToLong(&value);
+
+	sf->length_adjust = value - spectseq->GetFrameLength(sframe,0,NULL);
 
 	for(ix=0; ix < N_KLATTP; ix++)
 	{
@@ -268,7 +276,7 @@ void SpectDisplay::RefreshDialogValues(int type)
 		Refresh();
 	}
 	else
-		RefreshFrame(sframe);
+		RefreshFrame(sframe);   // frame amplitude
 }
 
 
@@ -525,7 +533,6 @@ void SpectDisplay::OnKey(wxKeyEvent& event)
 	int keyfr;
 	int control;
 	int number;
-	double value;
 	SpectFrame *sf;
 	peak_t *pk;
 //	static short pk_inc1[N_PEAKS] = {2,2,2,5,5,5,5,5,5};
@@ -777,10 +784,6 @@ void SpectDisplay::OnKey(wxKeyEvent& event)
 		display = 3;
 		break;
 
-	case 0x1052:   // CTRL-R
-		value = sf->GetRms(spectseq->amplitude);
-		break;
-
 	case 0x1053:   // CTRL-S
 		Save(savepath);
 		break;
@@ -974,6 +977,7 @@ void MyFrame::OnNewWindow(wxCommandEvent& event)
 	SpectSeq *spectseq;
 	wxString leaf;
 	wxString pathload;
+	int width, height;
 
 	if(event.GetId() == MENU_SPECTRUM)
 		pathload = path_spectload;
@@ -1015,9 +1019,10 @@ void MyFrame::OnNewWindow(wxCommandEvent& event)
 		path_spectload2 = path.GetPath();
 
 	// Make another frame, containing a canvas
+	GetClientSize(&width, &height);
 	MyChild *subframe = new MyChild(myframe, _T("Spectrum"),
-                                      wxPoint(10, 10), wxSize(300, 300),
-                                      wxDEFAULT_FRAME_STYLE | wxMAXIMIZE |
+                                      wxPoint(10, 0), wxSize(500, height),
+                                      wxDEFAULT_FRAME_STYLE |
                                       wxNO_FULL_REPAINT_ON_RESIZE);
 
 	subframe->SetTitle(leaf);
@@ -1025,14 +1030,13 @@ void MyFrame::OnNewWindow(wxCommandEvent& event)
 	// Give it a status line
 	subframe->CreateStatusBar();
 
-	int width, height;
 	subframe->GetClientSize(&width, &height);
 	SpectDisplay *canvas = new SpectDisplay(subframe, wxPoint(0, 0), wxSize(width, height), spectseq);
 	canvas->savepath = filename;
    currentcanvas = canvas;
 
 	// Associate the menu bar with the frame
-	subframe->SetMenuBar(MakeMenu(1));
+	subframe->SetMenuBar(MakeMenu(1,translator->dictionary_name));
 	subframe->canvas = canvas;
 	subframe->Show(TRUE);
 
