@@ -28,11 +28,13 @@
 #include <math.h>
 
 
+
 #include "speak_lib.h"
 #include "speech.h"
 #include "phoneme.h"
 #include "synthesize.h"
 #include "voice.h"
+#include "sonic.h"
 
 //#undef INCLUDE_KLATT
 
@@ -78,12 +80,11 @@ static wavegen_peaks_t peaks[N_PEAKS];
 static int peak_harmonic[N_PEAKS];
 static int peak_height[N_PEAKS];
 
-#define N_ECHO_BUF 5500   // max of 250mS at 22050 Hz
-static int echo_head;
-static int echo_tail;
+int echo_head;
+int echo_tail;
+int echo_amp = 0;
+short echo_buf[N_ECHO_BUF];
 static int echo_length = 0;   // period (in sample\) to ensure completion of echo at the end of speech, set in WavegenSetEcho()
-static int echo_amp = 0;
-static short echo_buf[N_ECHO_BUF];
 
 static int voicing;
 static RESONATOR rbreath[N_PEAKS];
@@ -134,8 +135,8 @@ int wcmdq_head=0;
 int wcmdq_tail=0;
 
 // pitch,speed,
-int embedded_default[N_EMBEDDED_VALUES]        = {0,50,170,100,50, 0,0, 0,170,0,0,0,0,0};
-static int embedded_max[N_EMBEDDED_VALUES]     = {0,0x7fff,600,300,99,99,99, 0,600,0,0,0,0,4};
+int embedded_default[N_EMBEDDED_VALUES]        = {0,    50,175,100,50, 0, 0, 0,175,0,0,0,0,0,0};
+static int embedded_max[N_EMBEDDED_VALUES]     = {0,0x7fff,750,300,99,99,99, 0,750,0,0,0,0,4,0};
 
 #define N_CALLBACK_IX N_WAV_BUF-2   // adjust this delay to match display with the currently spoken word
 int current_source_index=0;
@@ -149,35 +150,8 @@ static PortAudioStream *pa_stream=NULL;
 static PaStream *pa_stream=NULL;
 #endif
 
-/* default pitch envelope, a steady fall */
-#define ENV_LEN  128
-
-
-/*
-unsigned char Pitch_env0[ENV_LEN] = {
-    255,253,251,249,247,245,243,241,239,237,235,233,231,229,227,225,
-    223,221,219,217,215,213,211,209,207,205,203,201,199,197,195,193,
-    191,189,187,185,183,181,179,177,175,173,171,169,167,165,163,161,
-    159,157,155,153,151,149,147,145,143,141,139,137,135,133,131,129,
-    127,125,123,121,119,117,115,113,111,109,107,105,103,101, 99, 97,
-     95, 93, 91, 89, 87, 85, 83, 81, 79, 77, 75, 73, 71, 69, 67, 65,
-     63, 61, 59, 57, 55, 53, 51, 49, 47, 45, 43, 41, 39, 37, 35, 33,
-     31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11,  9,  7,  5,  3,  1
-};
-*/
-
-/*
-unsigned char Pitch_long[ENV_LEN] = {
-	254,249,250,251,252,253,254,254, 255,255,255,255,254,254,253,252,
-	251,250,249,247,244,242,238,234, 230,225,221,217,213,209,206,203,
-	199,195,191,187,183,179,175,172, 168,165,162,159,156,153,150,148,
-	145,143,140,138,136,134,132,130, 128,126,123,120,117,114,111,107,
-	104,100,96,91, 86,82,77,73, 70,66,63,60, 58,55,53,51,
-	49,47,46,45, 43,42,40,38, 36,34,31,28, 26,24,22,20,
-	18,16,14,12, 11,10,9,8, 8,8,8,8, 9,8,8,8,
-	8,8,7,7, 6,6,6,5, 4,4,3,3, 2,1,1,0
-};
-*/
+static sonicStream sonicSpeedupStream = NULL;
+double sonicSpeed = 1.0;
 
 // 1st index=roughness
 // 2nd index=modulation_type
@@ -285,12 +259,13 @@ unsigned char pitch_adjust_tab[MAX_PITCH_VALUE+1] = {
    217,220,223,226,229,232,236,239,
    242,246,249,252, 254,255 };
 
-int WavegenFill(int fill_zeros);
-
 
 #ifdef LOG_FRAMES
-static void LogMarker(int type, int value)
-{//=======================================
+static void LogMarker(int type, int value, int value2)
+{//===================================================
+	char buf[20];
+	int *p;
+
 	if(option_log_frames == 0)
 		return;
 
@@ -300,7 +275,13 @@ static void LogMarker(int type, int value)
 		if(f_log)
 		{
 			if(type == espeakEVENT_PHONEME)
-				fprintf(f_log,"Phoneme [%s]\n",WordToString(value));
+			{
+				p = (int *)buf;
+				p[0] = value;
+				p[1] = value2;
+				buf[8] = 0;
+				fprintf(f_log,"Phoneme [%s]\n", buf);
+			}
 			else
 				fprintf(f_log,"\n");
 			fclose(f_log);
@@ -314,9 +295,16 @@ void WcmdqStop()
 {//=============
 	wcmdq_head = 0;
 	wcmdq_tail = 0;
+	if(sonicSpeedupStream != NULL)
+	{
+		sonicDestroyStream(sonicSpeedupStream);
+		sonicSpeedupStream = NULL;
+	}
 #ifdef USE_PORTAUDIO
 	Pa_AbortStream(pa_stream);
 #endif
+	if(mbrola_name[0] != 0)
+		MbrolaReset();
 }
 
 
@@ -440,6 +428,10 @@ static int userdata[4];
 static PaError pa_init_err=0;
 static int out_channels=1;
 
+unsigned char *outbuffer = NULL;
+int outbuffer_size = 0;
+
+
 #if USE_PORTAUDIO == 18
 static int WaveCallback(void *inputBuffer, void *outputBuffer,
 		unsigned long framesPerBuffer, PaTimestamp outTime, void *userData )
@@ -452,9 +444,30 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	int ix;
 	int result;
 	unsigned char *p;
+	unsigned char *out_buf;
+	unsigned char *out_end2;
+	int pa_size;
 
-	out_ptr = out_start = (unsigned char *)outputBuffer;
-	out_end = out_ptr + framesPerBuffer*2;
+	pa_size = framesPerBuffer*2;
+
+	// make a buffer 3x size of the portaudio output
+	ix = pa_size*3;
+	if(ix > outbuffer_size)
+	{
+		outbuffer = (unsigned char *)realloc(outbuffer, ix);
+		if(outbuffer == NULL)
+		{
+			fprintf(stderr, "espeak: out of memory\n");
+		}
+		outbuffer_size = ix;
+		out_ptr = NULL;
+	}
+	if(out_ptr == NULL)
+	{
+		out_ptr = out_start = outbuffer;
+		out_end = out_start + outbuffer_size;
+	}
+	out_end2 = &outbuffer[pa_size];  // top of data needed for the portaudio buffer
 
 #ifdef LIBRARY
 	event_list_ix = 0;
@@ -462,6 +475,24 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 
 	result = WavegenFill(1);
 
+	// copy from the outbut buffer into the portaudio buffer
+	if(result && (out_ptr > out_end2))
+	{
+		result = 0;   // don't end yet, there is more data in the buffer than can fit in portaudio
+	}
+
+	while(out_ptr < out_end2)
+		*out_ptr++ = 0;  // fill with zeros up to the size of the portaudio buffer
+
+	memcpy(outputBuffer, outbuffer, pa_size);
+
+	// move the remaining contents of the start of the output buffer
+	for(p = out_end2; p < out_end; p++)
+	{
+		p[-pa_size] = p[0];
+	}
+	out_ptr -= pa_size;
+
 #ifdef LIBRARY
 	count_samples += framesPerBuffer;
 	if(synth_callback)
@@ -482,14 +513,15 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	{
 		// swap the order of bytes in each sound sample in the portaudio buffer
 		int c;
-		out_ptr = (unsigned char *)outputBuffer;
-		out_end = out_ptr + framesPerBuffer*2;
-		while(out_ptr < out_end)
+		unsigned char *buf_end;
+		out_buf = (unsigned char *)outputBuffer;
+		buf_end = out_buf + framesPerBuffer*2;
+		while(out_buf < buf_end)
 		{
-			c = out_ptr[0];
-			out_ptr[0] = out_ptr[1];
-			out_ptr[1] = c;
-			out_ptr += 2;
+			c = out_buf[0];
+			out_buf[0] = out_buf[1];
+			out_buf[1] = c;
+			out_buf += 2;
 		}
 	}
 #endif
@@ -498,12 +530,12 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	{
 		// sound output can only do stereo, not mono.  Duplicate each sound sample to
 		// produce 2 channels.
-		out_ptr = (unsigned char *)outputBuffer;
+		out_buf = (unsigned char *)outputBuffer;
 		for(ix=framesPerBuffer-1; ix>=0; ix--)
 		{
-			p = &out_ptr[ix*4];
-			p[3] = p[1] = out_ptr[ix*2 + 1];
-			p[2] = p[0] = out_ptr[ix*2];
+			p = &out_buf[ix*4];
+			p[3] = p[1] = out_buf[ix*2 + 1];
+			p[2] = p[0] = out_buf[ix*2];
 		}
 	}
 
@@ -720,6 +752,7 @@ void WavegenInit(int rate, int wavemult_fact)
 	max_hval = 0;
 
 	wdata.amplitude = 32;
+	wdata.amplitude_fmt = 100;
 
 	for(ix=0; ix<N_EMBEDDED_VALUES; ix++)
 		embedded_value[ix] = embedded_default[ix];
@@ -1174,7 +1207,8 @@ int Wavegen()
 				maxh2 = PeaksToHarmspect(peaks, wdata.pitch<<4, hspect[0], 0);
 
 				// adjust amplitude to compensate for fewer harmonics at higher pitch
-				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+//				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+				amplitude2 = (wdata.amplitude * (wdata.pitch >> 8) * wdata.amplitude_fmt)/(10000 << 3);
 
             // switch sign of harmonics above about 900Hz, to reduce max peak amplitude
 				h_switch_sign = 890 / (wdata.pitch >> 12);
@@ -1227,7 +1261,8 @@ int Wavegen()
 				}
 
 				// adjust amplitude to compensate for fewer harmonics at higher pitch
-				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+//				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+				amplitude2 = (wdata.amplitude * (wdata.pitch >> 8) * wdata.amplitude_fmt)/(10000 << 3);
 
 				if(glottal_flag > 0)
 				{
@@ -1412,11 +1447,12 @@ static int PlaySilence(int length, int resume)
 	static int n_samples;
 	int value=0;
 
-	if(length == 0)
-		return(0);
-
 	nsamples = 0;
 	samplecount = 0;
+	wavephase = 0x7fffffff;
+
+	if(length == 0)
+		return(0);
 
 	if(resume==0)
 		n_samples = length;
@@ -1513,14 +1549,36 @@ static int SetWithRange0(int value, int max)
 }
 
 
+static void SetPitchFormants()
+{//===========================
+	int ix;
+	int factor;
+	int pitch_value;
+
+	// adjust formants to give better results for a different voice pitch
+	if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
+		pitch_value = MAX_PITCH_VALUE;
+
+	if(pitch_value > 50)
+	{
+		// only adjust if the pitch is higher than normal
+		factor = 256 + (25 * (pitch_value - 50))/50;
+		for(ix=0; ix<=5; ix++)
+		{
+			wvoice->freq[ix] = (wvoice->freq2[ix] * factor)/256;
+		}
+	}
+	factor = embedded_value[EMBED_T]*3;
+	wvoice->height[0] = (wvoice->height2[0] * (256 - factor*2))/256;
+	wvoice->height[1] = (wvoice->height2[1] * (256 - factor))/256;
+}
+
+
 void SetEmbedded(int control, int value)
 {//=====================================
 	// there was an embedded command in the text at this point
 	int sign=0;
 	int command;
-	int ix;
-	int factor;
-	int pitch_value;
 
 	command = control & 0x1f;
 	if((control & 0x60) == 0x60)
@@ -1543,25 +1601,14 @@ void SetEmbedded(int control, int value)
 	case EMBED_T:
 		WavegenSetEcho();   // and drop through to case P
 	case EMBED_P:
-		// adjust formants to give better results for a different voice pitch
-		if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
-			pitch_value = MAX_PITCH_VALUE;
-
-		factor = 256 + (25 * (pitch_value - 50))/50;
-		for(ix=0; ix<=5; ix++)
-		{
-			wvoice->freq[ix] = (wvoice->freq2[ix] * factor)/256;
-		}
-		factor = embedded_value[EMBED_T]*3;
-		wvoice->height[0] = (wvoice->height2[0] * (256 - factor*2))/256;
-		wvoice->height[1] = (wvoice->height2[1] * (256 - factor))/256;
+		SetPitchFormants();
 		break;
 
 	case EMBED_A:  // amplitude
 		general_amplitude = GetAmplitude();
 		break;
 
-	case EMBED_F:   // emphasiis
+	case EMBED_F:   // emphasis
 		general_amplitude = GetAmplitude();
 		break;
 
@@ -1591,6 +1638,9 @@ void WavegenSetVoice(voice_t *v)
 		option_harmonic1 = 6;
 	}
 	WavegenSetEcho();
+	SetPitchFormants();
+	MarkerEvent(espeakEVENT_SAMPLERATE, 0, wvoice->samplerate, 0, out_ptr);
+//	WVoiceChanged(wvoice);
 }
 
 
@@ -1635,8 +1685,8 @@ void SetPitch2(voice_t *voice, int pitch1, int pitch2, int *pitch_base, int *pit
 	// compensate for change in pitch when the range is narrowed or widened
 	base -= (range - voice->pitch_range)*18;
 
-	*pitch_base = base + (pitch1 * range);
-	*pitch_range = base + (pitch2 * range) - *pitch_base;
+	*pitch_base = base + (pitch1 * range)/2;
+	*pitch_range = base + (pitch2 * range)/2 - *pitch_base;
 }
 
 
@@ -1767,7 +1817,7 @@ if(option_log_frames)
 		next = (fr2->fheight[ix] * v->height[ix]) << 6;
 		peaks[ix].height_inc =  ((next - peaks[ix].height1) * STEPSIZE) / length2;
 
-		if(ix <= wvoice->n_harmonic_peaks)
+		if((ix <= 5) && (ix <= wvoice->n_harmonic_peaks))
 		{
 			peaks[ix].left1 = (fr1->fwidth[ix] * v->width[ix]) << 10;
 			peaks[ix].left = int(peaks[ix].left1);
@@ -1812,8 +1862,7 @@ void Write4Bytes(FILE *f, int value)
 
 
 
-
-int WavegenFill(int fill_zeros)
+int WavegenFill2(int fill_zeros)
 {//============================
 // Pick up next wavegen commands from the queue
 // return: 0  output buffer has been filled
@@ -1822,14 +1871,10 @@ int WavegenFill(int fill_zeros)
 	long *q;
 	int length;
 	int result;
+	int marker_type;
 	static int resume=0;
 	static int echo_complete=0;
 
-#ifdef TEST_MBROLA
-	if(mbrola_name[0] != 0)
-		return(MbrolaFill(fill_zeros));
-#endif
-
 	while(out_ptr < out_end)
 	{
 		if(WcmdqUsed() <= 0)
@@ -1854,7 +1899,7 @@ int WavegenFill(int fill_zeros)
 		q = wcmdq[wcmdq_head];
 		length = q[1];
 
-		switch(q[0])
+		switch(q[0] & 0xff)
 		{
 		case WCMD_PITCH:
 			SetPitch(length,(unsigned char *)q[2],q[3] >> 16,q[3] & 0xffff);
@@ -1866,6 +1911,7 @@ int WavegenFill(int fill_zeros)
 				echo_complete -= length;
 			}
 			wdata.n_mix_wavefile = 0;
+			wdata.amplitude_fmt = 100;
 			KlattReset(1);
 			result = PlaySilence(length,resume);
 			break;
@@ -1910,13 +1956,14 @@ int WavegenFill(int fill_zeros)
 #endif
 
 		case WCMD_MARKER:
-			MarkerEvent(q[1],q[2],q[3],out_ptr);
+			marker_type = q[0] >> 8;
+			MarkerEvent(marker_type, q[1],q[2],q[3],out_ptr);
 #ifdef LOG_FRAMES
-			LogMarker(q[1],q[3]);
+			LogMarker(marker_type, q[2], q[3]);
 #endif
-			if(q[1] == 1)
+			if(marker_type == 1)  // word marker
 			{
-				current_source_index = q[2] & 0xffffff;
+				current_source_index = q[1] & 0xffffff;
 			}
 			break;
 
@@ -1932,6 +1979,19 @@ int WavegenFill(int fill_zeros)
 		case WCMD_EMBEDDED:
 			SetEmbedded(q[1],q[2]);
 			break;
+
+		case WCMD_MBROLA_DATA:
+			result = MbrolaFill(length, resume, (general_amplitude * wvoice->voicing)/64);
+			break;
+
+		case WCMD_FMT_AMPLITUDE:
+			if((wdata.amplitude_fmt = q[1]) == 0)
+				wdata.amplitude_fmt = 100;  // percentage, but value=0 means 100%
+			break;
+
+		case WCMD_SONIC_SPEED:
+			sonicSpeed = (double)q[1] / 1024;
+			break;
 		}
 
 		if(result==0)
@@ -1946,6 +2006,59 @@ int WavegenFill(int fill_zeros)
 	}
 
 	return(0);
+}  // end of WavegenFill2
+
+
+/* Speed up the audio samples with libsonic. */
+static int SpeedUp(short *outbuf, int length_in, int length_out, int end_of_text)
+{//==============================================================================
+	if(length_in >0)
+	{
+		if(sonicSpeedupStream == NULL)
+		{
+			sonicSpeedupStream = sonicCreateStream(22050, 1);
+		}
+		if(sonicGetSpeed(sonicSpeedupStream) != sonicSpeed)
+		{
+		        sonicSetSpeed(sonicSpeedupStream, sonicSpeed);
+		}
+	
+		sonicWriteShortToStream(sonicSpeedupStream, outbuf, length_in);
+	}
+
+	if(sonicSpeedupStream == NULL)
+		return(0);
+
+	if(end_of_text)
+	{
+		sonicFlushStream(sonicSpeedupStream);
+	}
+	return sonicReadShortFromStream(sonicSpeedupStream, outbuf, length_out);
+}  // end of SpeedUp
+
+
+/* Call WavegenFill2, and then speed up the output samples. */
+int WavegenFill(int fill_zeros)
+{//============================
+	int finished;
+	unsigned char *p_start;
+	int length;
+	int max_length;
+
+	p_start = out_ptr;
+
+	// fill_zeros is ignored. It is now done in the portaudio callback
+	finished = WavegenFill2(0);
+
+	if(sonicSpeed > 1.0)
+	{
+		max_length = (out_end - p_start);
+		length =  2*SpeedUp((short *)p_start, (out_ptr-p_start)/2, max_length/2, finished);
+		out_ptr = p_start + length;
+
+		if(length >= max_length)
+			finished = 0;   // there may be more data to flush
+	}
+	return finished;
 }  // end of WavegenFill
 
-
